@@ -1,26 +1,25 @@
 <?php
 /**
- * Horde_ActiveSync::
- *
  * @license   http://www.horde.org/licenses/gpl GPLv2
  *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
+
 /**
- * Horde_ActiveSync:: The Horde ActiveSync server. Entry point for performing
- * all ActiveSync operations.
+ * The Horde ActiveSync server. Entry point for performing all ActiveSync
+ * operations.
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
  *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  *
@@ -51,7 +50,8 @@ class Horde_ActiveSync
     const TRUNCATION_6                          = 6;
     const TRUNCATION_7                          = 7;
     const TRUNCATION_8                          = 8;
-    const TRUNCATION_NONE                       = 9;
+    const TRUNCATION_9                          = 9;
+    const TRUNCATION_NONE                       = 9; // @deprecated
 
     /* FOLDERHIERARCHY */
     const FOLDERHIERARCHY_FOLDERS               = 'FolderHierarchy:Folders';
@@ -262,6 +262,7 @@ class Horde_ActiveSync
     const FILTERTYPE_6MONTHS                    = 7;
     const FILTERTYPE_INCOMPLETETASKS            = 8;
 
+    // @todo normalize to string values.
     const PROVISIONING_FORCE                    = true;
     const PROVISIONING_LOOSE                    = 'loose';
     const PROVISIONING_NONE                     = false;
@@ -289,7 +290,7 @@ class Horde_ActiveSync
     const AUTH_REASON_USER_DENIED               = 'user';
     const AUTH_REASON_DEVICE_DENIED             = 'device';
 
-    const LIBRARY_VERSION                       = '2.19.2';
+    const LIBRARY_VERSION                       = '2.28.6';
 
     /**
      * Logger
@@ -303,7 +304,7 @@ class Horde_ActiveSync
      *
      * @var Horde_Log_Logger
      */
-    static protected $_logger;
+    protected static $_logger;
 
     /**
      * Provisioning support
@@ -324,7 +325,7 @@ class Horde_ActiveSync
      *
      * @var float
      */
-    static protected $_version;
+    protected static $_version;
 
     /**
      * Multipart support?
@@ -358,7 +359,7 @@ class Horde_ActiveSync
      *
      * @var Horde_ActiveSync_Device
      */
-    static protected $_device;
+    protected static $_device;
 
     /**
      * Wbxml encoder
@@ -395,12 +396,19 @@ class Horde_ActiveSync
      */
     protected $_procid;
 
+    /**
+     * Flag to  indicate we need to update the device version.
+     *
+     * @var boolean
+     */
+    protected $_needMsRp = false;
+
     /**
      * Supported EAS versions.
      *
      * @var array
      */
-    static protected $_supportedVersions = array(
+    protected static $_supportedVersions = array(
         self::VERSION_TWOFIVE,
         self::VERSION_TWELVE,
         self::VERSION_TWELVEONE,
@@ -417,7 +425,7 @@ class Horde_ActiveSync
      * @return Horde_ActiveSync_Message_Base   The concrete message object.
      * @todo For H6, move to Horde_ActiveSync_Message_Base::factory()
      */
-    static public function messageFactory($message)
+    public static function messageFactory($message)
     {
         $class = 'Horde_ActiveSync_Message_' . $message;
         if (!class_exists($class)) {
@@ -675,9 +683,9 @@ class Horde_ActiveSync
             $cmd = $get['Cmd'];
         }
         if (empty($devId)) {
-            $devId = !empty($get['DeviceId']) ? strtoupper($get['DeviceId']) : null;
+            $devId = !empty($get['DeviceId']) ? Horde_String::upper($get['DeviceId']) : null;
         } else {
-            $devId = strtoupper($devId);
+            $devId = Horde_String::upper($devId);
         }
         $this->_setLogger($get);
 
@@ -707,13 +715,10 @@ class Horde_ActiveSync
             throw new Horde_Exception_AuthenticationFailure();
         }
 
-        // Set provisioning support now that we are authenticated.
-        $this->setProvisioning($this->_driver->getProvisioning());
-
         self::$_logger->info(sprintf(
             '[%s] %s request received for user %s',
             $this->_procid,
-            strtoupper($cmd),
+            Horde_String::upper($cmd),
             $this->_driver->getUser())
         );
 
@@ -736,6 +741,98 @@ class Horde_ActiveSync
         // EAS Version
         $version = $this->getProtocolVersion();
 
+        // Device. Even though versions of EAS > 12.1 are supposed to send
+        // EAS status codes back to indicate various errors in allowing a client
+        // to connect, we just throw an exception (thus causing a HTTP error
+        // code to be sent as in versions 12.1 and below). Until we refactor for
+        // Horde 6, we don't know the response type to wrap the status code in
+        // until we load the request handler, which requires we start to parse
+        // the WBXML stream and device information etc... This saves resources
+        // as well as keeps things cleaner until we refactor.
+        $device_result = $this->_handleDevice($devId);
+
+        // Don't bother with everything else if all we want are Options
+        if ($cmd == 'Options') {
+            $this->_doOptionsRequest();
+            $this->_driver->clearAuthentication();
+            return true;
+        }
+
+        // Set provisioning support now that we are authenticated.
+        $this->setProvisioning($this->_driver->getProvisioning(self::$_device));
+
+        // Read the initial Wbxml header
+        $this->_decoder->readWbxmlHeader();
+
+        // Support Multipart response for ITEMOPERATIONS requests?
+        $headers = $this->_request->getHeaders();
+        if ((!empty($headers['ms-asacceptmultipart']) && $headers['ms-asacceptmultipart'] == 'T') ||
+            !empty($get['AcceptMultiPart'])) {
+            $this->_multipart = true;
+            self::$_logger->info(sprintf(
+                '[%s] Requesting multipart data.',
+                $this->_procid)
+            );
+        }
+
+        // Load the request handler to handle the request
+        // We must send the EAS header here, since some requests may start
+        // output and be large enough to flush the buffer (e.g., GetAttachment)
+        // See Bug: 12486
+        $this->activeSyncHeader();
+        if ($cmd != 'GetAttachment') {
+            $this->contentTypeHeader();
+        }
+
+        // Should we announce a new version is available to the client?
+        if (!empty($this->_needMsRp)) {
+            self::$_logger->info(sprintf(
+                '[%s] Announcing X-MS-RP to client.',
+                $this->_procid)
+            );
+            header("X-MS-RP: ". $this->getSupportedVersions());
+        }
+
+        // @TODO: Look at getting rid of having to set the version in the driver
+        //        and get it from the device object for H6.
+        $this->_driver->setDevice(self::$_device);
+        $class = 'Horde_ActiveSync_Request_' . basename($cmd);
+        if (class_exists($class)) {
+            $request = new $class($this);
+            $request->setLogger(self::$_logger);
+            $result = $request->handle();
+            self::$_logger->info(sprintf(
+                '[%s] Maximum memory usage for ActiveSync request: %d bytes.',
+                $this->_procid,
+                memory_get_peak_usage(true))
+            );
+
+            return $result;
+        }
+
+        $this->_driver->clearAuthentication();
+        throw new Horde_ActiveSync_Exception_InvalidRequest(basename($cmd) . ' not supported.');
+    }
+
+    /**
+     * Handle device checks. Takes into account permissions and restrictions
+     * via various callback methods.
+     *
+     * @param  string $devId  The client provided device id.
+     *
+     * @return boolean  If EAS version is > 12.1 returns false on any type of
+     *                  failure in allowing the device to connect. Sets
+     *                  appropriate internal variables to indicate the type of
+     *                  error to return to the client. Failure on EAS version
+     *                  < 12.1 results in throwing exceptions. Otherwise, return
+     *                  true.
+     * @throws Horde_ActiveSync_Exception, Horde_Exception_AuthenticationFailure
+     */
+    protected function _handleDevice($devId)
+    {
+        $get = $this->getGetVars();
+        $version = $this->getProtocolVersion();
+
         // Does device exist AND does the user have an account on the device?
         if (!$this->_state->deviceExists($devId, $this->_driver->getUser())) {
             // Device might exist, but with a new (additional) user account
@@ -763,11 +860,9 @@ class Horde_ActiveSync
                         self::$_device->id,
                         self::$_device->user);
                     self::$_logger->err($msg);
-                    if ($version > self::VERSION_TWELVEONE) {
-                        $this->_globalError = $callback_ret;
-                    } else {
-                        throw new Horde_ActiveSync_Exception($msg);
-                    }
+                    // Always throw exception in place of status code since we
+                    // won't have a version number before the device is created.
+                    throw new Horde_Exception_AuthenticationFailure($msg, $callback_ret);
                 } else {
                     // Give the driver a chance to modify device properties.
                     if (is_callable(array($this->_driver, 'modifyDeviceCallback'))) {
@@ -792,7 +887,7 @@ class Horde_ActiveSync
             }
             if (self::$_device->version < $this->_maxVersion &&
                 self::$_device->needsVersionUpdate($this->getSupportedVersions())) {
-                $needMsRp = true;
+                $this->_needMsRp = true;
             }
 
             // Give the driver a chance to modify device properties.
@@ -801,6 +896,8 @@ class Horde_ActiveSync
             }
         }
 
+        // Save the device now that we know it is at least allowed to connect,
+        // or it has connected successfully at least once in the past.
         self::$_device->save();
         if (is_callable(array($this->_driver, 'deviceCallback'))) {
             $callback_ret = $this->_driver->deviceCallback(self::$_device);
@@ -811,9 +908,12 @@ class Horde_ActiveSync
                     self::$_device->user);
                 self::$_logger->err($msg);
                 if ($version > self::VERSION_TWELVEONE) {
+                    // Use a status code here, since the device has already
+                    // connected.
                     $this->_globalError = $callback_ret;
+                    return false;
                 } else {
-                    throw new Horde_ActiveSync_Exception($msg);
+                    throw new Horde_Exception_AuthenticationFailure($msg, $callback_ret);
                 }
             }
         }
@@ -826,74 +926,17 @@ class Horde_ActiveSync
             self::$_logger->err($msg);
             if ($version > self::VERSION_TWELVEONE) {
                 $this->_globalError = Horde_ActiveSync_Status::DEVICE_BLOCKED_FOR_USER;
+                return false;
             } else {
                 throw new Horde_ActiveSync_Exception($msg);
             }
         }
 
-        // Don't bother with everything else if all we want are Options
-        if ($cmd == 'Options') {
-            $this->_doOptionsRequest();
-            $this->_driver->clearAuthentication();
-            return true;
-        }
-
-        // Read the initial Wbxml header
-        $this->_decoder->readWbxmlHeader();
-
-        // Support Multipart response for ITEMOPERATIONS requests?
-        $headers = $this->_request->getHeaders();
-        if ((!empty($headers['ms-asacceptmultipart']) && $headers['ms-asacceptmultipart'] == 'T') ||
-            !empty($get['AcceptMultiPart'])) {
-            $this->_multipart = true;
-            self::$_logger->info(sprintf(
-                '[%s] Requesting multipart data.',
-                $this->_procid)
-            );
-        }
-
-        // Load the request handler to handle the request
-        // We must send the eas header here, since some requests may start
-        // output and be large enough to flush the buffer (e.g., GetAttachment)
-        // See Bug: 12486
-        $this->activeSyncHeader();
-        if ($cmd != 'GetAttachment') {
-            $this->contentTypeHeader();
-        }
-
-        // Should we announce a new version is available to the client?
-        if (!empty($needMsRp)) {
-            self::$_logger->info(sprintf(
-                '[%s] Announcing X-MS-RP to client.',
-                $this->_procid)
-            );
-            header("X-MS-RP: ". $this->getSupportedVersions());
-        }
-
-        // @TODO: Look at getting rid of having to set the version in the driver
-        //        and get it from the device object for H6.
-        $this->_driver->setDevice(self::$_device);
-        $class = 'Horde_ActiveSync_Request_' . basename($cmd);
-        if (class_exists($class)) {
-            $request = new $class($this);
-            $request->setLogger(self::$_logger);
-            $result = $request->handle();
-            self::$_logger->info(sprintf(
-                '[%s] Maximum memory usage for ActiveSync request: %d bytes.',
-                $this->_procid,
-                memory_get_peak_usage())
-            );
-
-            return $result;
-        }
-
-        $this->_driver->clearAuthentication();
-        throw new Horde_ActiveSync_Exception_InvalidRequest(basename($cmd) . ' not supported.');
+        return true;
     }
 
     /**
      * Send the MS_Server-ActiveSync header.
-     *
      */
     public function activeSyncHeader()
     {
@@ -921,7 +964,6 @@ class Horde_ActiveSync
 
     /**
      * Send the protocol versions header.
-     *
      */
     public function versionHeader()
     {
@@ -941,7 +983,6 @@ class Horde_ActiveSync
 
     /**
      * Send protocol commands header.
-     *
      */
     public function commandsHeader()
     {
@@ -970,7 +1011,6 @@ class Horde_ActiveSync
 
     /**
      * Send provision header
-     *
      */
     public function provisionHeader()
     {
@@ -1091,7 +1131,6 @@ class Horde_ActiveSync
 
     /**
      * Send the OPTIONS request response headers.
-     *
      */
     protected function _doOptionsRequest()
     {
@@ -1102,14 +1141,15 @@ class Horde_ActiveSync
 
     /**
      * Return the number of bytes corresponding to the requested trunction
-     * constant.
+     * constant. This applies to MIMETRUNCATION only.
      *
      * @param integer $truncation  The constant.
      *
      * @return integer|boolean  Either the size, in bytes, to truncate or
      *                          falso if no truncation.
+     * @since 2.20.0
      */
-    static public function getTruncSize($truncation)
+    public static function getMIMETruncSize($truncation)
     {
         switch($truncation) {
         case Horde_ActiveSync::TRUNCATION_ALL:
@@ -1129,7 +1169,45 @@ class Horde_ActiveSync
         case Horde_ActiveSync::TRUNCATION_7:
             return 102400;
         case Horde_ActiveSync::TRUNCATION_8:
-        case Horde_ActiveSync::TRUNCATION_NONE:
+            return false;
+        default:
+            return 1024; // Default to 1Kb
+        }
+    }
+
+    /**
+     * Return the number of bytes corresponding to the requested trunction
+     * constant.
+     *
+     * @param integer $truncation  The constant.
+     *
+     * @return integer|boolean  Either the size, in bytes, to truncate or
+     *                          falso if no truncation.
+     *
+     */
+    public function getTruncSize($truncation)
+    {
+        switch($truncation) {
+        case Horde_ActiveSync::TRUNCATION_ALL:
+            return 0;
+        case Horde_ActiveSync::TRUNCATION_1:
+            return 512;
+        case Horde_ActiveSync::TRUNCATION_2:
+            return 1024;
+        case Horde_ActiveSync::TRUNCATION_3:
+            return 2048;
+        case Horde_ActiveSync::TRUNCATION_4:
+            return 5120;
+        case Horde_ActiveSync::TRUNCATION_5:
+            return 10240;
+        case Horde_ActiveSync::TRUNCATION_6:
+            return 20480;
+        case Horde_ActiveSync::TRUNCATION_7:
+            return 51200;
+        case Horde_ActiveSync::TRUNCATION_8:
+            return 102400;
+        case Horde_ActiveSync::TRUNCATION_9:
+        case Horde_ActiveSync::TRUNCATION_NONE: // @deprecated
             return false;
         default:
             return 1024; // Default to 1Kb
