@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2013-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2013-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -20,7 +20,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  * @internal  Not intended for use outside of the ActiveSync library.
@@ -75,13 +75,6 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
      */
     protected $_synckeyCount = 0;
 
-    /**
-     * Count of confirmed collections calculated for PARTIAL sync.
-     *
-     * @var integer
-     */
-    protected $_confirmedCount = 0;
-
     /**
      * Global WINDOWSIZE
      *
@@ -257,17 +250,35 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
     public function getNewCollection()
     {
         return array(
-            'truncation' => Horde_ActiveSync::TRUNCATION_ALL,
             'clientids' => array(),
             'fetchids' => array(),
             'windowsize' => 100,
-            'conflict' => Horde_ActiveSync::CONFLICT_OVERWRITE_PIM,
-            'bodyprefs' => array(),
-            'mimesupport' => Horde_ActiveSync::MIME_SUPPORT_NONE,
-            'mimetruncation' => Horde_ActiveSync::TRUNCATION_8,
+            'soft' => false,
+            'conflict' => Horde_ActiveSync::CONFLICT_OVERWRITE_PIM
         );
     }
 
+    /**
+     * Ensure default OPTIONS values are populated, while not overwriting any
+     * existing values.
+     *
+     * @since 2.20.0
+     */
+    public function ensureOptions()
+    {
+        foreach ($this->_collections as &$collection) {
+            $this->_logger->info(sprintf(
+                '[%s] Loading default OPTIONS for %s collection.', $this->_procid, $collection['id']));
+
+            if (!isset($collection['mimesupport'])) {
+                $collection['mimesupport'] = Horde_ActiveSync::MIME_SUPPORT_NONE;
+            }
+            if (!isset($collection['bodyprefs'])) {
+                $collection['bodyprefs'] = array();
+            }
+        }
+    }
+
     /**
      * Add a new populated collection array to the sync cache.
      *
@@ -712,6 +723,41 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         $this->_cache->clearCollectionKeys();
     }
 
+    /**
+     * Prepare the syncCache for an EMPTY sync request.
+     *
+     * @return boolean  False if EMPTY request cannot be performed, otherwise
+     *                  true.
+     * @since 2.25.0
+     */
+    public function initEmptySync()
+    {
+        $this->loadCollectionsFromCache();
+        foreach ($this->_collections as $value) {
+            // Remove keys from confirmed synckeys array and count them
+            if (isset($value['synckey'])) {
+                if (isset($this->_cache->confirmed_synckeys[$value['synckey']])) {
+                    $this->_logger->info(sprintf(
+                        'Removed %s from confirmed_synckeys',
+                        $value['synckey'])
+                    );
+                    $this->_cache->removeConfirmedKey($value['synckey']);
+                }
+                $this->_synckeyCount++;
+            }
+        }
+        if (!$this->_checkConfirmedKeys()) {
+            $this->_logger->err('Some synckeys were not confirmed, but handling an empty request. Requesting full SYNC');
+            $this->save();
+            return false;
+        }
+        $this->shortSyncRequest = true;
+        $this->hangingSync = true;
+        $this->save(true);
+
+        return true;
+    }
+
     /**
      * Prepares the syncCache for a partial sync request and checks that
      * it is allowed.
@@ -720,10 +766,38 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
      */
     public function initPartialSync()
     {
+        // PARTIAL is allowed without a <collection> tag if the waitinterval,
+        // heartbeat, or windowsize changed. So, short circuit the logic for
+        // checking for changed collections in this case.
         if (empty($this->_collections)) {
-            $this->_logger->err('No collections in collection handler, no PARTIAL allowed.');
-            return false;
+            $this->_logger->info('No collections in collection handler, loading full collection set from cache.');
+            $this->loadCollectionsFromCache();
+
+            // Need this for all PARTIAL sync requests.
+            $this->_tempSyncCache = clone $this->_cache;
+
+            foreach ($this->_collections as $value) {
+                // Remove keys from confirmed synckeys array and count them
+                if (isset($value['synckey'])) {
+                    if (isset($this->_cache->confirmed_synckeys[$value['synckey']])) {
+                        $this->_logger->info(sprintf(
+                            'Removed %s from confirmed_synckeys',
+                            $value['synckey'])
+                        );
+                        $this->_cache->removeConfirmedKey($value['synckey']);
+                    }
+                    $this->_synckeyCount++;
+                }
+            }
+            if (!$this->_checkConfirmedKeys()) {
+                $this->_logger->err('Some synckeys were not confirmed. Requesting full SYNC');
+                $this->save();
+                return false;
+            }
+
+            return true;
         }
+
         $this->_tempSyncCache = clone $this->_cache;
         $c = $this->_tempSyncCache->getCollections();
         foreach ($this->_collections as $key => $value) {
@@ -769,11 +843,12 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
                     $this->_unchangedCount++;
                 }
                 // Unset in tempSyncCache, since we have it from device.
-                // Afterwards, anything left in tempSyncCache needs to be
-                // added to _collections.
+                // Afterwards, anything left in tempSyncCache needs to be added
+                // to _collections (this is done in
+                // self::_haveNoChangesInPartialSync()).
                 $this->_tempSyncCache->removeCollection($key);
 
-                // Remove keys from confirmed synckeys array and count them
+                // Remove keys from confirmed synckeys array and count them.
                 if (isset($value['synckey'])) {
                     if (isset($this->_cache->confirmed_synckeys[$value['synckey']])) {
                         $this->_logger->info(sprintf(
@@ -781,20 +856,13 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
                             $value['synckey'])
                         );
                         $this->_cache->removeConfirmedKey($value['synckey']);
-                        $this->_confirmedCount++;
                     }
                     $this->_synckeyCount++;
                 }
             }
         }
 
-        $csk = $this->_cache->confirmed_synckeys;
-        if ($csk) {
-            $this->_logger->info(sprintf(
-                '[%s] Confirmed Synckeys contains %s',
-                $this->_procid,
-                serialize($csk))
-            );
+        if (!$this->_checkConfirmedKeys()) {
             $this->_logger->err('Some synckeys were not confirmed. Requesting full SYNC');
             $this->save();
             return false;
@@ -810,6 +878,21 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         return true;
     }
 
+    protected function _checkConfirmedKeys()
+    {
+        $csk = $this->_cache->confirmed_synckeys;
+        if ($csk) {
+            $this->_logger->info(sprintf(
+                '[%s] Confirmed Synckeys contains %s',
+                $this->_procid,
+                serialize($csk))
+            );
+            return false;
+        }
+
+        return true;
+    }
+
     /**
      * Return if we can do an empty response
      *
@@ -879,11 +962,10 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         if (!empty($cc[$id]['filtertype']) &&
             !is_null($filter) &&
             $cc[$id]['filtertype'] != $filter) {
-
-            $this->_cache->removeCollection($id, true);
-            $this->_cache->save();
-            $this->_logger->info('Invalidating SYNCKEY and removing collection - found updated filtertype');
-
+            $this->_logger->info(sprintf(
+                '[%s] Filtertype change from: %d to %d',
+                $this->_procid, $cc[$id]['filtertype'], $filter));
+            $this->_cache->updateFiltertype($id, $filter);
             return false;
         }
 
@@ -1219,13 +1301,15 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
     }
 
     /**
-     * Return the any changes for the current collection, and cache them if
-     * we are not PINGing.
+     * Return any changes for the current collection.
      *
      * @param boolean $ping  True if this is a PING request, false otherwise.
+     *                       If true, we only detect that a change has occured,
+     *                       not the data on all of the changes.
      * @param array $ensure  An array of UIDs that should be sent in the
      *                       current response if possible, and not put off
      *                       because of a MOREAVAILABLE situation.
+     *                       @deprecated and no longer used.
      *
      * @return array  The changes array.
      */
@@ -1235,29 +1319,9 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
             $this->_changes = $this->_as->state->getChanges(array('ping' => $ping));
         }
 
-        if (!empty($ensure)) {
-            $this->_changes = $this->_reorderChanges($ensure);
-        }
-
         return $this->_changes;
     }
 
-    protected function _reorderChanges(array $ensure)
-    {
-        $changes = array();
-        foreach ($this->_changes as $change) {
-            if (array_search($change['id'], $ensure) !== false) {
-                $this->_logger->info(sprintf(
-                    'Placing %s at beginning of changes array.', $change['id']));
-                array_unshift($changes, $change);
-            } else {
-                $changes[] = $change;
-            }
-        }
-
-        return $changes;
-    }
-
     /**
      * Return the count of the current collection's chagnes.
      *
