@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2013-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2013-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -19,7 +19,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  *
@@ -52,6 +52,7 @@ class Horde_ActiveSync_Device
     const OS                = 'Settings:OS';
     const OS_LANGUAGE       = 'Settings:OSLanguage';
     const PHONE_NUMBER      = 'Settings:PhoneNumber';
+    const OPERATOR          = 'Settings:MobileOperator';
     const VERSION           = 'version';
     const MULTIPLEX         = 'multiplex';
     const ANNOUNCED_VERSION = 'announcedVersion';
@@ -249,7 +250,7 @@ class Horde_ActiveSync_Device
     {
         // Outlook? The specs say that "Windows Communication Apps" should
         // provide the 'OS' parameter of the ITEMSETTINGS data equal to 'Windows',
-        // but Outlook 2013 doesn't even sent the ITEMSETTINGS command, so we
+        // but Outlook 2013 doesn't even send the ITEMSETTINGS command, so we
         // need to check the userAgent header. Early versions used Microsoft.Outlook,
         // but after some update it was changed to 'Outlook/15.0'
         if (strpos($this->deviceType, 'MicrosoftOutlook') !== false ||
@@ -314,6 +315,9 @@ class Horde_ActiveSync_Device
         if (!empty($this->properties[self::NAME])) {
             $data[_("Common Name")] = $this->properties[self::NAME];
         }
+        if (!empty($this->properties[self::OPERATOR])) {
+            $data[_("Mobile Operator")] = $this->properties[self::OPERATOR];
+        }
         if (!empty($this->properties[self::OS])) {
             $data[_("OS")] = $this->properties[self::OS];
         }
@@ -371,7 +375,7 @@ class Horde_ActiveSync_Device
      */
     public function getMajorVersion()
     {
-        switch (strtolower($this->clientType)) {
+        switch (Horde_String::lower($this->clientType)) {
             case self::TYPE_BLACKBERRY:
                 if (preg_match('/(.+)\/(.+)/', $this->userAgent, $matches)) {
                     return $matches[2];
@@ -421,7 +425,7 @@ class Horde_ActiveSync_Device
      */
     public function getMinorVersion()
     {
-        switch (strtolower($this->clientType)) {
+        switch (Horde_String::lower($this->clientType)) {
             case self::TYPE_BLACKBERRY:
                 if (preg_match('/(.+)\/(.+)/', $this->userAgent, $matches)) {
                     return $matches[2];
@@ -531,7 +535,7 @@ class Horde_ActiveSync_Device
      */
     public function normalizePoomContactsDates($date, $toEas = false)
     {
-        switch (strtolower($this->clientType)) {
+        switch (Horde_String::lower($this->clientType)) {
         case self::TYPE_WP:
         case 'wp8': // Legacy. Remove in H6.
         case 'wp':  // Legacy. Remove in H6.
@@ -543,7 +547,18 @@ class Horde_ActiveSync_Device
             }
 
         case self::TYPE_ANDROID:
-            if ($this->getMajorVersion() >= 4) {
+            // Need to protect against clients that don't send the actual Android
+            // version in the OS field.
+            if (stripos($this->deviceType, 'samsung') === 0) {
+                // Samsung's native Contacts app works differently than stock
+                // Android, always sending as 00:00:00
+                if ($toEas) {
+                    return new Horde_Date($date->format('Y-m-d'), 'UTC');
+                }
+                $date = new Horde_Date($date->format('Y-m-d'));
+                return $date->setTimezone('UTC');
+            }
+            if ($this->getMajorVersion() >= 4 && $this->getMajorVersion() <= 10) {
                 if ($toEas) {
                     return new Horde_Date($date->format('Y-m-d 08:00:00'), 'UTC');
                 } else {
@@ -607,7 +622,7 @@ class Horde_ActiveSync_Device
         // Differentiate between the deviceType and the client app.
         if ((!empty($this->properties[self::OS]) &&
              stripos($this->properties[self::OS], 'Android') !== false) ||
-             strtolower($this->deviceType) == self::TYPE_ANDROID) {
+             Horde_String::lower($this->deviceType) == self::TYPE_ANDROID) {
 
             // We can detect native Android, TouchDown, and Nine.
             // Moxier does not distinguish itself, so we can't sniff it.
@@ -635,6 +650,9 @@ class Horde_ActiveSync_Device
      */
     protected function _isNine()
     {
+        if (!ctype_xdigit($this->id)) {
+            return false;
+        }
         return stripos(pack('H*', $this->id), 'nine') === 0;
     }
 
@@ -645,14 +663,16 @@ class Horde_ActiveSync_Device
      */
     protected function _sniffMultiplex()
     {
-        $clientType = strtolower($this->clientType);
+        $clientType = Horde_String::lower($this->clientType);
         if (strpos($this->userAgent, 'iOS') === 0 || in_array($clientType, array(self::TYPE_IPAD, self::TYPE_IPOD, self::TYPE_IPHONE))) {
             // iOS seems to support multiple collections for everything except Notes.
             $this->_properties['properties'][self::MULTIPLEX] = Horde_ActiveSync_Device::MULTIPLEX_NOTES;
         } else if ($clientType == self::TYPE_ANDROID) {
-            // Special cases: Samsung's native EAS client doesn't support
-            // multiplex.
-            if (stripos($this->deviceType, 'samsung') === 0) {
+            // Special cases: These clients don't support non-multiplexed
+            // collections. Samsung's native client and HTCOnemini2.
+            if (stripos($this->deviceType, 'samsung') === 0 ||
+                stripos($this->model, 'HTCOnemini2') === 0 ||
+                $this->deviceType == 'HTCOnemini2') {
                 $this->_properties['properties'][self::MULTIPLEX] =
                     Horde_ActiveSync_Device::MULTIPLEX_CONTACTS |
                     Horde_ActiveSync_Device::MULTIPLEX_CALENDAR |
@@ -662,12 +682,11 @@ class Horde_ActiveSync_Device
                 return;
             }
 
-            // All android before 4.4 KitKat requires multiplex. KitKat supports
-            // non-multiplexed calendars only.
+            // All android before 4.4 KitKat requires multiplex. KitKat and
+            // Android 5 native supports non-multiplexed calendars only.
             if (!empty($this->properties[self::OS]) &&
-                preg_match('/(\d+\.\d+\.\d+)/', $this->properties[self::OS], $matches) &&
+                preg_match('/(\d+\.\d+(\.\d+)*)/', $this->properties[self::OS], $matches) &&
                 version_compare($matches[0], '4.4.0') >= 0) {
-
                 $this->_properties['properties'][self::MULTIPLEX] =
                     Horde_ActiveSync_Device::MULTIPLEX_NOTES |
                     Horde_ActiveSync_Device::MULTIPLEX_CONTACTS |
@@ -679,10 +698,14 @@ class Horde_ActiveSync_Device
                     Horde_ActiveSync_Device::MULTIPLEX_NOTES |
                     Horde_ActiveSync_Device::MULTIPLEX_TASKS;
             }
+        } else if (strpos($this->userAgent, 'MSFT-WP/8.10') !== false) {
+            // Windows Phone 8.10 supports multiple calendars and tasks, but
+            // no contacts.
+            $this->_properties['properties'][self::MULTIPLEX] =
+                Horde_ActiveSync_Device::MULTIPLEX_CONTACTS;
         } else if (strpos($this->userAgent, 'MSFT-WP/8.0') !== false || $this->deviceType == 'WP8') {
-            // Windows Phone. For the devices I've tested, it seems that
-            // only multiple tasklists are accepted. The rest must be
-            // multiplexed.
+            // Windows Phone 8.0 seems that only multiple tasklists are
+            // supported. The rest must be multiplexed.
             $this->_properties['properties'][self::MULTIPLEX] =
                 Horde_ActiveSync_Device::MULTIPLEX_CONTACTS |
                 Horde_ActiveSync_Device::MULTIPLEX_CALENDAR;
