@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -22,7 +22,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -128,7 +128,7 @@ abstract class Horde_ActiveSync_Driver_Base
             $this->_logger = $params['logger'];
             unset($params['logger']);
         } else {
-            $this->_logger = new Horde_Support_Stub;
+            $this->_logger = new Horde_Log_Logger(new Horde_Log_Handler_Null());
         }
 
         $this->_state = $params['state'];
@@ -232,7 +232,8 @@ abstract class Horde_ActiveSync_Driver_Base
     }
 
     /**
-     * Obtain a message from the backend.
+     * Obtain the FULL message from the backend, regardless of any truncation
+     * options.
      *
      * @param string $folderid   Folder id containing data to fetch.
      * @param string $id         Server id of data to fetch.
@@ -242,8 +243,10 @@ abstract class Horde_ActiveSync_Driver_Base
      */
     public function fetch($folderid, $id, array $collection)
     {
-        // Forces entire message
-        $collection['truncation'] = 0;
+        // Forces entire message since FETCH requests are part of normal SYNC
+        // requests and, as such, will have the default OPTIONS loaded which
+        // should not applied to FETCH requests.
+        $collection['truncation'] = $collection['mimetruncation'] = false;
         if (!empty($collection['bodyprefs'])) {
             foreach ($collection['bodyprefs'] as &$bodypref) {
                 if (isset($bodypref['truncationsize'])) {
@@ -259,27 +262,15 @@ abstract class Horde_ActiveSync_Driver_Base
      *
      * @param array $bodyprefs  BODYPREFERENCE data.
      *
-     * @return array  THe BODYPREFERENCE data, with default truncationsize values.
+     * @return array  The BODYPREFERENCE data, with default truncationsize values.
+     * @deprecated  Will be removed in 3.0. We don't need and indeed shouldn't
+     *              set default values for truncationsize. Not to mention we
+     *              were setting the incorrect constant value instead of the
+     *              byte value.
+     * @todo
      */
     public function addDefaultBodyPrefTruncation(array $bodyprefs)
     {
-        if (isset($bodyprefs[Horde_ActiveSync::BODYPREF_TYPE_PLAIN]) &&
-            !isset($bodyprefs[Horde_ActiveSync::BODYPREF_TYPE_PLAIN]['truncationsize'])) {
-            $bodyprefs[Horde_ActiveSync::BODYPREF_TYPE_PLAIN]['truncationsize'] = 1048576; // 1024 * 1024
-        }
-        if (isset($bodyprefs[Horde_ActiveSync::BODYPREF_TYPE_HTML]) &&
-            !isset($bodyprefs[Horde_ActiveSync::BODYPREF_TYPE_HTML]['truncationsize'])) {
-            $bodyprefs[Horde_ActiveSync::BODYPREF_TYPE_HTML]['truncationsize'] = 1048576; // 1024 * 1024
-        }
-        if (isset($bodyprefs[Horde_ActiveSync::BODYPREF_TYPE_RTF]) &&
-            !isset($bodyprefs[Horde_ActiveSync::BODYPREF_TYPE_RTF]['truncationsize'])) {
-            $bodyprefs[Horde_ActiveSync::BODYPREF_TYPE_RTF]['truncationsize'] = 1048576; // 1024 * 1024
-        }
-        if (isset($bodyprefs[Horde_ActiveSync::BODYPREF_TYPE_MIME]) &&
-            !isset($bodyprefs[Horde_ActiveSync::BODYPREF_TYPE_MIME]['truncationsize'])) {
-            $bodyprefs[Horde_ActiveSync::BODYPREF_TYPE_MIME]['truncationsize'] = 1048576; // 1024 * 1024
-        }
-
         return $bodyprefs;
     }
 
@@ -315,7 +306,7 @@ abstract class Horde_ActiveSync_Driver_Base
      * @return string   The EAS freebusy string.
      * @since 2.4.0
      */
-    static public function buildFbString($fb, Horde_Date $start, Horde_Date $end)
+    public static function buildFbString($fb, Horde_Date $start, Horde_Date $end)
     {
         if (empty($fb)) {
             return false;
@@ -591,6 +582,17 @@ abstract class Horde_ActiveSync_Driver_Base
      *
      * @return array A list of messge uids that have chnaged in the specified
      *               time period.
+     *
+     * @todo Horde 6
+     * - Change return structure to (optionally) include the actual $to_ts value
+     *   that was used. This is needed because if using something like
+     *   Kolab/IMAP storage backend in Horde, then we must synchronize
+     *   Horde <-> Kolab to get any changes from external Kolab clients and this
+     *   may cause new/changed entries that would change the current MODSEQ.
+     *   Not critical, since without this it only results in having to wait
+     *   until the next SYNC cycle. By returning the actual $to_ts value we can
+     *   update the state with THIS value instead of the one we were originally
+     *   sent.
      */
     abstract public function getServerChanges(
         $folderId, $from_ts, $to_ts, $cutoffdate, $ping);
@@ -611,14 +613,19 @@ abstract class Horde_ActiveSync_Driver_Base
      * @param string $folderid    The server's folder id this message is from
      * @param string $id          The server's message id
      * @param array  $collection  The colletion data. May contain things like:
-     *   - mimesupport: (boolean) Indicates if the device has MIME support.
-     *                  DEFAULT: false (No MIME support)
-     *   - truncation: (integer)  The truncation constant, if sent by the device.
+     *   - mimesupport: (integer) Indicates if the device has MIME support.
+     *                  DEFAULT: 0 (No MIME support)
+     *   - truncation: (integer)  Non-MIME truncation limit. Anything larger
+     *                            than this amount of bytes will be truncated.
      *                 DEFAULT: 0 (No truncation)
+     *   - mimetruncation: (integer)  MIME truncation limit. Anything larger
+     *                                than this amount of bytes will be
+     *                                truncated.
+     *                DEFAULT: 0 (No truncation)
      *   - bodyprefs: (array)  The bodypref array from the device.
      *
      * @return Horde_ActiveSync_Message_Base The message data
-     * @throws Horde_ActiveSync_Exception
+     * @throws Horde_ActiveSync_Exception, Horde_Exception_NotFound
      */
     abstract public function getMessage($folderid, $id, array $collection);
 
@@ -666,7 +673,7 @@ abstract class Horde_ActiveSync_Driver_Base
     abstract public function setReadFlag($folderid, $uid, $flag);
 
     /**
-     * Sends the email represented by the rfc822 string received by the PIM.
+     * Sends the email represented by the rfc822 string received by the client.
      *
      * @param mixed $rfc822     The rfc822 mime message, a string or stream
      *                          resource.
