@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -20,7 +20,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -52,7 +52,7 @@ abstract class Horde_ActiveSync_Folder_Base
      *
      * @var boolean
      */
-    public $haveInitialSync = true;
+    public $haveInitialSync = false;
 
     /**
      * Timestamp for the last sincedate used for SOFTDELETE.
