@@ -9,7 +9,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -22,7 +22,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -43,7 +43,7 @@ class Horde_ActiveSync_Folder_Imap extends Horde_ActiveSync_Folder_Base implemen
     /**
      * The folder's current message list.
      * Note: This represents the folder list on the client and is affected by
-     * the FILTER on the collection.
+     * the FILTERTYPE on the collection.
      *
      * @var array
      */
@@ -102,25 +102,47 @@ class Horde_ActiveSync_Folder_Imap extends Horde_ActiveSync_Folder_Base implemen
      */
     protected $_categories = array();
 
+    /**
+     * Internal flag to indicate initial first sync/prime.
+     *
+     * @var boolean
+     */
+    protected $_primed = false;
+
     /**
      * Set message changes.
      *
-     * @param array $messages    An array of message UIDs.
-     * @param array $flags       A hash of message read flags, keyed by UID.
-     * @param array $categories  A hash of custom message flags, keyed by UID.
-     *                           @since 2.17.0
+     * @param array $messages       An array of message UIDs.
+     * @param array $flags          A hash of message read flags, keyed by UID.
+     * @param array $categories     A hash of custom message flags, keyed by UID.
+     *                              @since 2.17.0
+     * @param boolean $resetMinUid  If true, reset the minimum UID. Should be
+     *                              used when FilterType has widened and
+     *                              messages older than originally selected are
+     *                              being returned. @since 2.24.0
      */
-    public function setChanges(array $messages, array $flags = array(), array $categories = array())
+    public function setChanges(
+        array $messages, array $flags = array(), array $categories = array(),
+        $resetMinUid = false)
     {
         $uidnext = $this->uidnext();
         $minuid = $this->minuid();
         $modseq = $this->modseq();
+
         foreach ($messages as $uid) {
             if ($uid >= $uidnext) {
+                // new (to us) message.
                 $this->_added[] = $uid;
-            } elseif ($uid >= $minuid) {
+            } elseif ($uid >= $minuid || $resetMinUid) {
                 if ($modseq > 0) {
-                    $this->_changed[] = $uid;
+                    // MODSEQ capable server, either a changed message, or
+                    // one that was previously outside of FilterType but is now
+                    // within.
+                    if (array_search($uid, $this->_messages) !== false) {
+                        $this->_changed[] = $uid;
+                    } else {
+                        $this->_added[] = $uid;
+                    }
                 } else {
                     if (empty($this->_messages[$uid])) {
                         // Do not know about this message
@@ -151,6 +173,20 @@ class Horde_ActiveSync_Folder_Imap extends Horde_ActiveSync_Folder_Base implemen
         }
     }
 
+    /**
+     * Sets initial message collection for servers that support MODSEQ. Much
+     * more effecient than using setChanges() when we know this is the initial
+     * folder "priming".
+     *
+     * @param array $messages  Array of message UIDs for the initial message
+     *                         set for this folder.
+     */
+    public function primeFolder($messages)
+    {
+        $this->_added = $messages;
+        $this->_primed = true;
+    }
+
     /**
      * Return a list of message uids that should be SOFTDELETEd from the client.
      * Must be called after setChanges and setRemoved, but before updateState.
@@ -243,10 +279,15 @@ class Horde_ActiveSync_Folder_Imap extends Horde_ActiveSync_Folder_Base implemen
             }
             $this->_messages = $this->_flags + array_flip($this->_messages);
         } else {
-            foreach ($this->_added as $add) {
-                $this->_messages[] = $add;
+            if ($this->_primed) {
+                $this->_messages = $this->_added;
+                $this->_primed = false;
+            } else {
+                foreach ($this->_added as $add) {
+                    $this->_messages[] = $add;
+                }
+                $this->_messages = array_diff($this->_messages, $this->_removed);
             }
-            $this->_messages = array_diff($this->_messages, $this->_removed);
         }
 
         // Clean up
@@ -255,6 +296,7 @@ class Horde_ActiveSync_Folder_Imap extends Horde_ActiveSync_Folder_Base implemen
         $this->_changed = array();
         $this->_flags = array();
         $this->_softDeleted = array();
+        $this->haveInitialSync = true;
     }
 
     /**
@@ -435,6 +477,7 @@ class Horde_ActiveSync_Folder_Imap extends Horde_ActiveSync_Folder_Base implemen
         $this->_class = $d_data['c'];
         $this->_lastSinceDate = $d_data['lsd'];
         $this->_softDelete = $d_data['sd'];
+        $this->haveInitialSync = !empty($this->_messages);
 
         if (!empty($this->_status[self::HIGHESTMODSEQ]) && is_string($this->_messages)) {
             $this->_messages = $this->_fromSequenceString($this->_messages);
