@@ -1,16 +1,15 @@
 <?php
 /**
- * Horde_ActiveSync_Imap_Message::
- *
  * @license   http://www.horde.org/licenses/gpl GPLv2
  *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
+
 /**
  * This class provides all functionality related to parsing and working with
  * a single IMAP email message when using Horde_Imap_Client.
@@ -22,9 +21,15 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
+ *
+ * @property-read Horde_Imap_Client_Data_Envelope $envelope
+ *     The message envelope.
+ * @property-read array $flags                     The message flags.
+ * @property-read integer $uid                     The message uid.
+ * @property-read Horde_ActiveSync_Mime $basePart  The base message part.
  */
 class Horde_ActiveSync_Imap_Message
 {
@@ -40,7 +45,7 @@ class Horde_ActiveSync_Imap_Message
      *
      * @var Horde_ActiveSync_Mime
      */
-    protected $_message;
+    protected $_basePart;
 
     /**
      * The imap client.
@@ -49,13 +54,6 @@ class Horde_ActiveSync_Imap_Message
      */
     protected $_imap;
 
-    /**
-     * The envelope data.
-     *
-     * @var Horde_Imap_Client_Data_Envelope
-     */
-    protected $_envelope;
-
     /**
      * Cache if the last body part was encoded or not.
      *
@@ -85,12 +83,40 @@ class Horde_ActiveSync_Imap_Message
         Horde_Imap_Client_Data_Fetch $data)
     {
         $this->_imap = $imap;
-        $this->_message = new Horde_ActiveSync_Mime($data->getStructure());
-        $this->_uid = $data->getUid();
-        $this->_flags = $data->getFlags();
         $this->_mbox = $mbox;
         $this->_data = $data;
-        $this->_envelope = $data->getEnvelope();
+    }
+
+    public function __destruct()
+    {
+        $this->_data = null;
+        $this->_basePart = null;
+    }
+
+    /**
+     * Accessor
+     *
+     * @param  string $property The property.
+     *
+     * @return mixed
+     */
+    public function &__get($property)
+    {
+        switch ($property) {
+        case 'envelope':
+            return $this->_data->getEnvelope();
+        case 'flags':
+            return $this->_data->getFlags();
+        case 'uid':
+            return $this->_data->getUid();
+        case 'basePart':
+            if (empty($this->_basePart)) {
+                $this->_basePart = new Horde_ActiveSync_Mime($this->_data->getStructure());
+            }
+            return $this->_basePart;
+        }
+
+        throw new InvalidArgumentException(sprintf('The property %s of Horde_ActiveSync_Imap_Message does not exist', $property));
     }
 
     /**
@@ -159,19 +185,29 @@ class Horde_ActiveSync_Imap_Message
     public function getFullMsg($stream = false)
     {
         // First see if we already have it.
-        if (!$full = $this->_data->getFullMsg()) {
+        if ($stream) {
+            $full = new Horde_Stream_Existing(array('stream' => $this->_data->getFullMsg($stream)));
+            $length = $full->length();
+            if (!$length) {
+                $full->close();
+            }
+        } else {
+            $full = $this->_data->getFullMsg(false);
+            $length = strlen($full);
+        }
+        if (!$length) {
             $query = new Horde_Imap_Client_Fetch_Query();
             $query->fullText(array('peek' => true));
             try {
                 $fetch_ret = $this->_imap->fetch(
                     $this->_mbox,
                     $query,
-                    array('ids' => new Horde_Imap_Client_Ids(array($this->_uid)))
+                    array('ids' => new Horde_Imap_Client_Ids(array($this->uid)))
                 );
             } catch (Horde_Imap_Exception $e) {
                 throw new Horde_ActiveSync_Exception($e);
             }
-            $data = $fetch_ret[$this->_uid];
+            $data = $fetch_ret[$this->uid];
             $full = $data->getFullMsg($stream);
         }
 
@@ -185,7 +221,7 @@ class Horde_ActiveSync_Imap_Message
      */
     public function getStructure()
     {
-        return $this->_message->base;
+        return $this->basePart->base;
     }
 
     /**
@@ -208,174 +244,37 @@ class Horde_ActiveSync_Imap_Message
      */
     public function getMessageBodyData(array $options = array())
     {
-        $version = empty($options['protocolversion']) ?
-            Horde_ActiveSync::VERSION_TWOFIVE :
-            $options['protocolversion'];
+        return $this->getMessageBodyDataObject($options)->toArray();
+    }
 
-        // Look for the parts we need. We try to detect and fetch only the parts
-        // we need, while ensuring we have something to return. So, e.g., if we
-        // don't have BODYPREF_TYPE_HTML, we only request plain text, but if we
-        // can't find plain text but we have a html body, fetch that anyway.
-        $text_id = $this->_message->findBody('plain');
-        $html_id = $this->_message->findBody('html');
-
-        // Deduce which part(s) we need to request.
-        $want_html_text = $version >= Horde_ActiveSync::VERSION_TWELVE &&
-            (!empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_HTML]) ||
-            !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_MIME]));
-
-        $want_plain_text = $version == Horde_ActiveSync::VERSION_TWOFIVE ||
-            empty($options['bodyprefs']) ||
-            !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_PLAIN]) ||
-            !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_RTF]) ||
-            !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_MIME]) ||
-            ($want_html_text && empty($html_id));
-
-        $want_html_as_plain = false;
-        if (!empty($text_id) && $want_plain_text) {
-            $text_body_part = $this->_message->getPart($text_id);
-            $charset = $text_body_part->getCharset();
-        } elseif ($want_plain_text && !empty($html_id) &&
-                  empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_MIME])) {
-            $want_html_text = true;
-            $want_html_as_plain = true;
-        }
-        if (!empty($html_id) && $want_html_text) {
-            $html_body_part = $this->_message->getPart($html_id);
-            $html_charset = $html_body_part->getCharset();
-        }
-
-        // Sanity check the truncation stuff
-        if (empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_PLAIN]) &&
-            !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_HTML]) &&
-            $want_plain_text && $want_html_text) {
-
-            // We only have HTML truncation data, requested HTML body but only
-            // have plaintext.
-            $options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_PLAIN] = $options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_HTML];
-        }
-
-        $query = new Horde_Imap_Client_Fetch_Query();
-        $query_opts = array(
-            'decode' => true,
-            'peek' => true
+    /**
+     * Returns the main text body of the message suitable for sending over
+     * EAS response.
+     *
+     * @param array $options  An options array containgin:
+     *  - bodyprefs: (array)  Bodypref settings
+     *               DEFAULT: none (No bodyprefs used).
+     *  - mimesupport: (integer)  Indicates if MIME is supported or not.
+     *                  Possible values: 0 - Not supported 1 - Only S/MIME or
+     *                  2 - All MIME.
+     *                  DEFAULT: 0 (No MIME support)
+     *  - protocolversion: (float)  The EAS protocol we are supporting.
+     *                     DEFAULT 2.5
+     *
+     * @return Horde_ActiveSync_Imap_MessageBodyData  The result.
+     *
+     * @throws Horde_ActiveSync_Exception, Horde_Exception_NotFound
+     */
+    public function getMessageBodyDataObject(array $options = array())
+    {
+        return new Horde_ActiveSync_Imap_MessageBodyData(
+            array(
+                'imap' => $this->_imap,
+                'mbox' => $this->_mbox,
+                'uid' => $this->uid,
+                'mime' => $this->basePart),
+            $options
         );
-
-        // Get body information
-        if ($version >= Horde_ActiveSync::VERSION_TWELVE) {
-            if (!empty($html_id)) {
-                $query->bodyPartSize($html_id);
-                $query->bodyPart($html_id, $query_opts);
-            }
-            if (!empty($text_id)) {
-                $query->bodyPart($text_id, $query_opts);
-                $query->bodyPartSize($text_id);
-            }
-        } else {
-            // EAS 2.5 Plaintext body
-            $query->bodyPart($text_id, $query_opts);
-            $query->bodyPartSize($text_id);
-        }
-        try {
-            $fetch_ret = $this->_imap->fetch(
-                $this->_mbox,
-                $query,
-                array('ids' => new Horde_Imap_Client_Ids(array($this->_uid)))
-            );
-        } catch (Horde_Imap_Client_Exception $e) {
-            throw new Horde_ActiveSync_Exception($e);
-        }
-        if (!$data = $fetch_ret->first()) {
-            throw new Horde_Exception_NotFound(
-                sprintf('Could not load message %s from server.', $this->_uid));
-        }
-
-        $return = array();
-        if (!empty($text_id) && $want_plain_text) {
-            $text = $data->getBodyPart($text_id);
-            if (!$data->getBodyPartDecode($text_id)) {
-                $text_body_part->setContents($text);
-                $text = $text_body_part->getContents();
-            }
-
-            $text_size = !is_null($data->getBodyPartSize($text_id))
-                    ? $data->getBodyPartSize($text_id)
-                    : Horde_String::length($text);
-
-            if (!empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_PLAIN]['truncationsize'])) {
-                // EAS >= 12.0 truncation
-                $text = Horde_String::substr($text, 0, $options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_PLAIN]['truncationsize'], $charset);
-            }
-
-            $truncated = $text_size > Horde_String::length($text);
-            if ($version >= Horde_ActiveSync::VERSION_TWELVE &&
-                $truncated && !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_PLAIN]['allornone'])) {
-                $text = '';
-            }
-            $return ['plain'] = array(
-                'charset' => $charset,
-                'body' => $text,
-                'truncated' => $truncated,
-                'size' => $text_size);
-        }
-
-        if (!empty($html_id) && $want_html_text) {
-            $html = $data->getBodyPart($html_id);
-            if (!$data->getBodyPartDecode($html_id)) {
-                $html_body_part->setContents($html);
-                $html = $html_body_part->getContents();
-            }
-
-            // Size of the original HTML part.
-            $html_size = !is_null($data->getBodyPartSize($html_id))
-                ? $data->getBodyPartSize($html_id)
-                : Horde_String::length($html);
-
-            if (!empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_HTML]['truncationsize'])) {
-                $html = Horde_String::substr(
-                    $html,
-                    0,
-                    $options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_HTML]['truncationsize'],
-                    $html_charset);
-            } elseif ($want_html_as_plain) {
-                $html = Horde_Text_Filter::filter(
-                    $html, 'Html2text', array('charset' => $html_charset));
-
-                // Get the new size, since it probably changed.
-                $html_size = Horde_String::length($html);
-                if (!empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_PLAIN]['truncationsize'])) {
-                    // EAS >= 12.0 truncation
-                    $html = Horde_String::substr(
-                        $html,
-                        0,
-                        $options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_PLAIN]['truncationsize'],
-                        $html_charset);
-                }
-            }
-
-            // Was the part truncated?
-            $truncated = $html_size > Horde_String::length($html);
-
-            if ($want_html_as_plain) {
-                $return['plain'] = array(
-                    'charset' => $html_charset,
-                    'body' => $html,
-                    'truncated' => $truncated,
-                    'size' => $html_size
-                );
-            }
-
-            if ($version >= Horde_ActiveSync::VERSION_TWELVE &&
-                !($truncated && !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_HTML]['allornone']))) {
-                $return['html'] = array(
-                    'charset' => $html_charset,
-                    'body' => $html,
-                    'estimated_size' => $html_size,
-                    'truncated' => $truncated);
-            }
-        }
-
-        return $return;
     }
 
     /**
@@ -389,13 +288,14 @@ class Horde_ActiveSync_Imap_Message
     public function getAttachments($version)
     {
         $ret = array();
-        $map = $this->_message->contentTypeMap();
+        $map = $this->basePart->contentTypeMap();
         foreach ($map as $id => $type) {
             if ($this->isAttachment($id, $type)) {
                 if ($type != 'application/ms-tnef' || (!$mime_part = $this->_decodeTnefData($id))) {
                     $mime_part = $this->getMimePart($id, array('nocontents' => true));
                 }
                 $ret[] = $this->_buildEasAttachmentFromMime($id, $mime_part, $version);
+                $mime_part = null;
             }
         }
 
@@ -423,10 +323,10 @@ class Horde_ActiveSync_Imap_Message
             $atc->attoid = $mime_part->getContentId();
         }
         $atc->attsize = intval($mime_part->getBytes(true));
-        $atc->attname = $this->_mbox . ':' . $this->_uid . ':' . $id;
+        $atc->attname = $this->_mbox . ':' . $this->uid . ':' . $id;
         $atc->displayname = Horde_String::convertCharset(
             $this->getPartName($mime_part, true),
-            $this->_message->getHeaderCharset(),
+            $this->basePart->getHeaderCharset(),
             'UTF-8',
             true);
         $atc->attmethod = in_array($mime_part->getType(), array('message/rfc822', 'message/disposition-notification'))
@@ -492,7 +392,7 @@ class Horde_ActiveSync_Imap_Message
     public function getAttachmentsMimeParts()
     {
         $mime_parts = array();
-        $map = $this->_message->contentTypeMap();
+        $map = $this->basePart->contentTypeMap();
         foreach ($map as $id => $type) {
             if ($this->isAttachment($id, $type)) {
                 $mpart = $this->getMimePart($id);
@@ -500,7 +400,9 @@ class Horde_ActiveSync_Imap_Message
                 if ($mpart->getPrimaryType() == 'text') {
                     $mpart->setContents(
                         $mpart->replaceEOL(
-                            $mpart->getContents(array('stream' => true)), Horde_Mime_Part::RFC_EOL, true)
+                            $mpart->getContents(array('stream' => true)), Horde_Mime_Part::RFC_EOL, true
+                        ),
+                        array('usestream' => true)
                     );
                 }
                 if ($mpart->getType() == 'text/calendar') {
@@ -511,6 +413,8 @@ class Horde_ActiveSync_Imap_Message
                     $part = $mpart;
                 }
                 $mime_parts[] = $part;
+                $mpart = null;
+                $part = null;
             }
         }
 
@@ -528,11 +432,11 @@ class Horde_ActiveSync_Imap_Message
      *   - nocontents: (boolean) If true, don't add the contents to the part
      *                 DEFAULT: Contents are added to the part
      *
-     * @return Horde_Mime_Part  The raw MIME part asked for (reference).
+     * @return Horde_Mime_Part  The raw MIME part asked for.
      */
     public function getMimePart($id, array $options = array())
     {
-        $part = $this->_message->getPart($id);
+        $part = $this->basePart->getPart($id);
         if ($part &&
             (strcasecmp($part->getCharset(), 'ISO-8859-1') === 0)) {
             $part->setCharset('windows-1252');
@@ -551,7 +455,6 @@ class Horde_ActiveSync_Imap_Message
                     'length' => empty($options['length']) ? null : $options['length'],
                     'stream' => true)
             );
-
             $part->setContents($body, array('encoding' => $this->_lastBodyPartDecode, 'usestream' => true));
         }
 
@@ -637,6 +540,7 @@ class Horde_ActiveSync_Imap_Message
      *
      * @return mixed  The text of the part or a stream resource if 'stream'
      *                is true.
+     * @todo Simplify by removing 'mimeheaders' parameter (not used).
      */
     public function getBodyPart($id, $options)
     {
@@ -671,19 +575,19 @@ class Horde_ActiveSync_Imap_Message
         $fetch_res = $this->_imap->fetch(
             $this->_mbox,
             $query,
-            array('ids' => new Horde_Imap_Client_Ids(array($this->_uid)))
+            array('ids' => new Horde_Imap_Client_Ids(array($this->uid)))
         );
 
         if (empty($options['mimeheaders'])) {
-            $this->_lastBodyPartDecode = $fetch_res[$this->_uid]->getBodyPartDecode($id);
-            return $fetch_res[$this->_uid]->getBodyPart($id, $options['stream']);
+            $this->_lastBodyPartDecode = $fetch_res[$this->uid]->getBodyPartDecode($id);
+            return $fetch_res[$this->uid]->getBodyPart($id, $options['stream']);
         } elseif (empty($options['stream'])) {
-            return $fetch_res[$this->_uid]->getMimeHeader($id) . $fetch_res[$this->_uid]->getBodyPart($id);
+            return $fetch_res[$this->uid]->getMimeHeader($id) . $fetch_res[$this->uid]->getBodyPart($id);
         } else {
-            $swrapper = new Horde_Support_CombineSream(
+            $swrapper = new Horde_Support_CombineStream(
                 array(
-                    $fetch_res[$this->_uid]->getMimeHeader($id, Horde_Imap_Client_Data_Fetch::HEADER_STREAM),
-                    $fetch_res[$this->_uid]->getBodyPart($id, true))
+                    $fetch_res[$this->uid]->getMimeHeader($id, Horde_Imap_Client_Data_Fetch::HEADER_STREAM),
+                    $fetch_res[$this->uid]->getBodyPart($id, true))
             );
 
             return $swrapper->fopen();
@@ -698,7 +602,7 @@ class Horde_ActiveSync_Imap_Message
      */
     public function getToAddresses()
     {
-        $to = $this->_envelope->to;
+        $to = $this->envelope->to;
         $dtos = $tos = array();
         foreach ($to->raw_addresses as $e) {
             $tos[] = $e->bare_address;
@@ -712,10 +616,15 @@ class Horde_ActiveSync_Imap_Message
      * Return the CC addresses for this message.
      *
      * @return string  The Cc address string.
+     * @throws Horde_ActiveSync_Exception @since 2.27.0
      */
     public function getCc()
     {
-        $cc = new Horde_Mail_Rfc822_List($this->_envelope->cc->addresses);
+        try {
+            $cc = new Horde_Mail_Rfc822_List($this->envelope->cc->addresses);
+        } catch (Horde_Mail_Exception $e) {
+            throw new Horde_ActiveSync_Exception($e);
+        }
         return $cc->writeAddress();
     }
 
@@ -723,11 +632,16 @@ class Horde_ActiveSync_Imap_Message
      * Return the ReplyTo Address
      *
      * @return string
+     * @throws Horde_ActiveSync_Exception @since 2.27.0
      */
     public function getReplyTo()
     {
-        $r = $this->_envelope->reply_to->addresses;
-        $a = new Horde_Mail_Rfc822_Address(current($r));
+        $r = $this->envelope->reply_to->addresses;
+        try {
+            $a = new Horde_Mail_Rfc822_Address(current($r));
+        } catch (Horde_Mail_Exception $e) {
+            throw new Horde_ActiveSync_Exception($e);
+        }
 
         return $a->writeAddress(false);
     }
@@ -736,11 +650,16 @@ class Horde_ActiveSync_Imap_Message
      * Return the message's From: address.
      *
      * @return string  The From address of this message.
+     * @throws Horde_ActiveSync_Exception @since 2.27.0
      */
     public function getFromAddress()
     {
-        $from = $this->_envelope->from->addresses;
-        $a = new Horde_Mail_Rfc822_Address(current($from));
+        $from = $this->envelope->from->addresses;
+        try {
+            $a = new Horde_Mail_Rfc822_Address(current($from));
+        } catch (Horde_ActiveSync_Exception $e) {
+            throw new Horde_ActiveSync_Exception($e);
+        }
 
         return $a->writeAddress(false);
     }
@@ -752,7 +671,7 @@ class Horde_ActiveSync_Imap_Message
      */
     public function getSubject()
     {
-        return $this->_envelope->subject;
+        return $this->envelope->subject;
     }
 
     /**
@@ -762,7 +681,7 @@ class Horde_ActiveSync_Imap_Message
      */
     public function getDate()
     {
-        return new Horde_Date((string)$this->_envelope->date);
+        return new Horde_Date((string)$this->envelope->date);
     }
 
     /**
@@ -774,7 +693,7 @@ class Horde_ActiveSync_Imap_Message
      */
     public function getFlag($flag)
     {
-        return (array_search($flag, $this->_flags) !== false)
+        return (array_search($flag, $this->flags) !== false)
             ? 1
             : 0;
     }
@@ -787,7 +706,7 @@ class Horde_ActiveSync_Imap_Message
      */
     public function getFlags()
     {
-        return $this->_flags;
+        return $this->flags;
     }
 
     /**
@@ -798,7 +717,7 @@ class Horde_ActiveSync_Imap_Message
      */
     public function contentTypeMap()
     {
-        return $this->_message->contentTypeMap();
+        return $this->basePart->contentTypeMap();
     }
 
     /**
@@ -815,7 +734,7 @@ class Horde_ActiveSync_Imap_Message
      */
     public function isAttachment($id, $mime_type)
     {
-        return $this->_message->isAttachment($id, $mime_type);
+        return $this->basePart->isAttachment($id, $mime_type);
     }
 
     /**
@@ -825,7 +744,16 @@ class Horde_ActiveSync_Imap_Message
      */
     public function hasiCalendar()
     {
-        return $this->_message->hasiCalendar();
+        if ($id = $this->basePart->hasiCalendar()) {
+            // May already have downloaded the part.
+            $part = $this->basePart->base->getPart($id);
+            if (!$part->getContents(array('stream' => true))) {
+                return $this->getMimePart($id);
+            }
+            return $part;
+        }
+
+        return false;
     }
 
     /**
@@ -835,17 +763,16 @@ class Horde_ActiveSync_Imap_Message
      */
     public function hasAttachments()
     {
-        return $this->_message->hasAttachments();
+        return $this->basePart->hasAttachments();
     }
 
     /**
-     * Return the S/MIME status of this message (RFC2633)
+     * Return the S/MIME signature status of this message (RFC2633)
      *
      * @param Horde_Mime_Part $message  A mime part to check. If omitted, use
-     *                                  $this->_message.
+     *                                  self::$_message.
      *
-     * @return boolean True if message is S/MIME signed or encrypted,
-     *                 false otherwise.
+     * @return boolean True if message is S/MIME signed, false otherwise.
      */
     public function isSigned(Horde_Mime_Part $message = null)
     {
@@ -854,7 +781,26 @@ class Horde_ActiveSync_Imap_Message
             return $message->isSigned();
         }
 
-        return $this->_message->isSigned();
+        return $this->basePart->isSigned();
+    }
+
+    /**
+     * Return the S/MIME encryption status of this message (RFC2633)
+     *
+     * @param Horde_Mime_Part $message  A mime part to check. If omitted, use
+     *                                  self::$_message.
+     *
+     * @return boolean True if message is S/MIME signed or encrypted,
+     *                 false otherwise.
+     */
+    public function isEncrypted(Horde_Mime_Part $message = null)
+    {
+        if (!empty($message)) {
+            $message = new Horde_ActiveSync_Mime($message);
+            return $message->isEncrypted();
+        }
+
+        return $this->basePart->isEncrypted();
     }
 
 }
