@@ -18,7 +18,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -31,22 +31,26 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
 class Horde_ActiveSync_Message_Base
 {
     /* Attribute Keys */
-    const KEY_ATTRIBUTE    = 1;
-    const KEY_VALUES       = 2;
-    const KEY_TYPE         = 3;
+    const KEY_ATTRIBUTE         = 1;
+    const KEY_VALUES            = 2;
+    const KEY_TYPE              = 3;
+    const KEY_PROPERTY          = 4;
 
     /* Types */
-    const TYPE_DATE        = 1;
-    const TYPE_HEX         = 2;
-    const TYPE_DATE_DASHES = 3;
-    const TYPE_MAPI_STREAM = 4;
+    const TYPE_DATE             = 1;
+    const TYPE_HEX              = 2;
+    const TYPE_DATE_DASHES      = 3;
+    const TYPE_MAPI_STREAM      = 4;
+    const TYPE_MAPI_GOID        = 5;
+
+    const PROPERTY_NO_CONTAINER = 7;
 
     /**
      * Holds the mapping for object properties
@@ -105,6 +109,8 @@ class Horde_ActiveSync_Message_Base
      */
     protected $_device;
 
+    protected $_streamFilters = array();
+
     /**
      * Const'r
      *
@@ -122,7 +128,7 @@ class Horde_ActiveSync_Message_Base
         if (!empty($options['logger'])) {
             $this->_logger = $options['logger'];
         } else {
-            $this->_logger = new Horde_Support_Stub();
+            $this->_logger =  new Horde_Log_Logger(new Horde_Log_Handler_Null());
         }
         if (!empty($options['protocolversion'])) {
             $this->_version = $options['protocolversion'];
@@ -132,6 +138,13 @@ class Horde_ActiveSync_Message_Base
         }
     }
 
+    public function __destruct()
+    {
+        foreach ($this->_streamFilters as $filter) {
+            stream_filter_remove($filter);
+        }
+    }
+
     /**
      * Return the EAS version this object supports.
      *
@@ -288,9 +301,9 @@ class Horde_ActiveSync_Message_Base
                 if (!($entity[Horde_ActiveSync_Wbxml::EN_FLAGS] & Horde_ActiveSync_Wbxml::EN_FLAGS_CONTENT)) {
                     $map = $this->_mapping[$entity[Horde_ActiveSync_Wbxml::EN_TAG]];
                     if (!isset($map[self::KEY_TYPE])) {
-                        $this->$map[self::KEY_ATTRIBUTE] = '';
+                        $this->{$map[self::KEY_ATTRIBUTE]} = '';
                     } elseif ($map[self::KEY_TYPE] == self::TYPE_DATE || $map[self::KEY_TYPE] == self::TYPE_DATE_DASHES ) {
-                        $this->$map[self::KEY_ATTRIBUTE] = '';
+                        $this->{$map[self::KEY_ATTRIBUTE]} = '';
                     }
                     continue;
                 }
@@ -304,8 +317,11 @@ class Horde_ActiveSync_Message_Base
                     if (isset($map[self::KEY_VALUES])) {
                         // Handle arrays of attribute values
                         while (1) {
-                            if (!$decoder->getElementStartTag($map[self::KEY_VALUES])) {
-                                break;
+                            //do not get start tag for an array without a container
+                            if (!(isset($map[self::KEY_PROPERTY]) && $map[self::KEY_PROPERTY] == self::PROPERTY_NO_CONTAINER)) {
+                                if (!$decoder->getElementStartTag($map[self::KEY_VALUES])) {
+                                    break;
+                                }
                             }
                             if (isset($map[self::KEY_TYPE])) {
                                 $class = $map[self::KEY_TYPE];
@@ -317,18 +333,33 @@ class Horde_ActiveSync_Message_Base
                             } else {
                                 $decoded = $decoder->getElementContent();
                             }
-                            if (!isset($this->$map[self::KEY_ATTRIBUTE])) {
-                                $this->$map[self::KEY_ATTRIBUTE] = array($decoded);
+                            if (!isset($this->{$map[self::KEY_ATTRIBUTE]})) {
+                                $this->{$map[self::KEY_ATTRIBUTE]} = array($decoded);
                             } else {
                                 $this->{$map[self::KEY_ATTRIBUTE]}[] = $decoded;
                             }
+
                             if (!$decoder->getElementEndTag()) {
                                 throw new Horde_ActiveSync_Exception('Missing expected wbxml end tag');
                             }
+                            if (isset($map[self::KEY_PROPERTY]) && $map[self::KEY_PROPERTY] == self::PROPERTY_NO_CONTAINER) {
+                                $e = $decoder->peek();
+                                //go back to the initial while if another block of no container elements is found
+                                if ($e[Horde_ActiveSync_Wbxml::EN_TYPE] == Horde_ActiveSync_Wbxml::EN_TYPE_STARTTAG) {
+                                    continue 2;
+                                }
+                                //break on end tag because no container elements block end is reached
+                                if ($e[Horde_ActiveSync_Wbxml::EN_TYPE] == Horde_ActiveSync_Wbxml::EN_TYPE_ENDTAG)
+                                    break;
+                                if (empty($e))
+                                    break;
+                            }
                         }
-
-                        if (!$decoder->getElementEndTag()) {
-                            return false;
+                        //do not get container end tag for an array without a container
+                        if (!(isset($map[self::KEY_PROPERTY]) && $map[self::KEY_PROPERTY] == self::PROPERTY_NO_CONTAINER)) {
+                            if (!$decoder->getElementEndTag()) {
+                                return false;
+                            }
                         }
                     } else {
                         // Handle a simple attribute value
@@ -336,45 +367,32 @@ class Horde_ActiveSync_Message_Base
                             // Complex type, decode recursively
                             if ($map[self::KEY_TYPE] == self::TYPE_DATE || $map[self::KEY_TYPE] == self::TYPE_DATE_DASHES) {
                                 $decoded = $this->_parseDate($decoder->getElementContent());
-                                if (!$decoder->getElementEndTag()) {
-                                    throw new Horde_ActiveSync_Exception('Missing expected wbxml end tag');
-                                }
                             } elseif ($map[self::KEY_TYPE] == self::TYPE_HEX) {
                                 $decoded = self::hex2bin($decoder->getElementContent());
-                                if (!$decoder->getElementEndTag()) {
-                                   throw new Horde_ActiveSync_Exception('Missing expected wbxml end tag');
-                                }
                             } else {
                                 $class = $map[self::KEY_TYPE];
                                 $subdecoder = new $class(array(
                                     'protocolversion' => $this->_version,
                                     'logger' => $this->_logger)
                                 );
-                                if ($subdecoder->decodeStream($decoder) === false) {
-                                    throw new Horde_ActiveSync_Exception('Missing expected wbxml end tag');
-                                }
+                                $subdecoder->decodeStream($decoder);
                                 $decoded = $subdecoder;
-                                if (!$decoder->getElementEndTag()) {
-                                    $this->_logger->err('No end tag for ' . $entity[Horde_ActiveSync_Wbxml::EN_TAG]);
-                                    throw new Horde_ActiveSync_Exception('Missing expected wbxml end tag');
-                                }
                             }
                         } else {
                             // Simple type, just get content
                             $decoded = $decoder->getElementContent();
                             if ($decoded === false) {
-                                $this->_logger->err('Unable to get content for ' . $entity[Horde_ActiveSync_Wbxml::EN_TAG]);
-                            }
-                            if (!$decoder->getElementEndTag()) {
-                                $this->_logger->err('Unable to get end tag for ' . $entity[Horde_ActiveSync_Wbxml::EN_TAG]);
-                                throw new Horde_ActiveSync_Exception('Missing expected wbxml end tag');
+                                $decoded = '';
+                                $this->_logger->notice('Unable to get expected content for ' . $entity[Horde_ActiveSync_Wbxml::EN_TAG] . ': Setting to an empty string.');
                             }
                         }
-                        // $decoded now contains data object (or string)
-                        $this->$map[self::KEY_ATTRIBUTE] = $decoded;
+                        if (!$decoder->getElementEndTag()) {
+                            $this->_logger->err('Unable to get end tag for ' . $entity[Horde_ActiveSync_Wbxml::EN_TAG]);
+                            throw new Horde_ActiveSync_Exception('Missing expected wbxml end tag');
+                        }
+                        $this->{$map[self::KEY_ATTRIBUTE]} = $decoded;
                     }
                 }
-
             } elseif ($entity[Horde_ActiveSync_Wbxml::EN_TYPE] == Horde_ActiveSync_Wbxml::EN_TYPE_ENDTAG) {
                 $decoder->_ungetElement($entity);
                 break;
@@ -394,19 +412,24 @@ class Horde_ActiveSync_Message_Base
     public function encodeStream(Horde_ActiveSync_Wbxml_Encoder &$encoder)
     {
         foreach ($this->_mapping as $tag => $map) {
-            if (isset($this->$map[self::KEY_ATTRIBUTE])) {
+            if (isset($this->{$map[self::KEY_ATTRIBUTE]})) {
                 // Variable is available
-                if (is_object($this->$map[self::KEY_ATTRIBUTE]) && !($this->$map[self::KEY_ATTRIBUTE] instanceof Horde_Date)) {
+                if (is_object($this->{$map[self::KEY_ATTRIBUTE]}) &&
+                    !($this->{$map[self::KEY_ATTRIBUTE]} instanceof Horde_Date)) {
                     // Subobjects can do their own encoding
                     $encoder->startTag($tag);
-                    $this->$map[self::KEY_ATTRIBUTE]->encodeStream($encoder);
+                    $this->{$map[self::KEY_ATTRIBUTE]}->encodeStream($encoder);
                     $encoder->endTag();
-                } elseif (isset($map[self::KEY_VALUES]) && is_array($this->$map[self::KEY_ATTRIBUTE])) {
+                } elseif (isset($map[self::KEY_VALUES]) &&
+                          is_array($this->{$map[self::KEY_ATTRIBUTE]})) {
                     // Array of objects. Note that some array values must be
                     // send as an empty tag if they contain no elements.
-                    if (count($this->$map[self::KEY_ATTRIBUTE])) {
-                        $encoder->startTag($tag);
-                        foreach ($this->$map[self::KEY_ATTRIBUTE] as $element) {
+                    if (count($this->{$map[self::KEY_ATTRIBUTE]})) {
+                        if (!isset($map[self::KEY_PROPERTY]) ||
+                            $map[self::KEY_PROPERTY] != self::PROPERTY_NO_CONTAINER) {
+                            $encoder->startTag($tag);
+                        }
+                        foreach ($this->{$map[self::KEY_ATTRIBUTE]} as $element) {
                             if (is_object($element)) {
                                 // Outputs object container (eg Attachment)
                                 $encoder->startTag($map[self::KEY_VALUES]);
@@ -421,17 +444,19 @@ class Horde_ActiveSync_Message_Base
                                 }
                             }
                         }
-                        $encoder->endTag();
+                        if (!isset($map[self::KEY_PROPERTY]) || $map[self::KEY_PROPERTY] != self::PROPERTY_NO_CONTAINER) {
+                            $encoder->endTag();
+                        }
                     } elseif ($this->_checkSendEmpty($tag)) {
                         $encoder->startTag($tag, null, true);
                     }
                 } else {
                     // Simple type
-                    if (!is_resource($this->$map[self::KEY_ATTRIBUTE]) &&
-                        strlen($this->$map[self::KEY_ATTRIBUTE]) == 0) {
+                    if (!is_resource($this->{$map[self::KEY_ATTRIBUTE]}) &&
+                        strlen($this->{$map[self::KEY_ATTRIBUTE]}) == 0) {
                           // Do not output empty items except for the following:
                           if ($this->_checkSendEmpty($tag)) {
-                              $encoder->startTag($tag, $this->$map[self::KEY_ATTRIBUTE], true);
+                              $encoder->startTag($tag, $this->{$map[self::KEY_ATTRIBUTE]}, true);
                           }
                           continue;
                     } elseif ($encoder->multipart &&
@@ -441,7 +466,7 @@ class Horde_ActiveSync_Message_Base
                                 Horde_ActiveSync_Request_ItemOperations::ITEMOPERATIONS_DATA)
                               )) {
                         $this->_logger->info('HANDLING MULTIPART OUTPUT');
-                        $encoder->addPart($this->$map[self::KEY_ATTRIBUTE]);
+                        $encoder->addPart($this->{$map[self::KEY_ATTRIBUTE]});
                         $encoder->startTag(Horde_ActiveSync_Request_ItemOperations::ITEMOPERATIONS_PART);
                         $encoder->content((string)(count($encoder->getParts()) - 1));
                         $encoder->endTag();
@@ -450,16 +475,16 @@ class Horde_ActiveSync_Message_Base
 
                     $encoder->startTag($tag);
                     if (isset($map[self::KEY_TYPE]) && ($map[self::KEY_TYPE] == self::TYPE_DATE || $map[self::KEY_TYPE] == self::TYPE_DATE_DASHES)) {
-                        if (!empty($this->$map[self::KEY_ATTRIBUTE])) { // don't output 1-1-1970
-                            $encoder->content($this->_formatDate($this->$map[self::KEY_ATTRIBUTE], $map[self::KEY_TYPE]));
+                        if (!empty($this->{$map[self::KEY_ATTRIBUTE]})) { // don't output 1-1-1970
+                            $encoder->content($this->_formatDate($this->{$map[self::KEY_ATTRIBUTE]}, $map[self::KEY_TYPE]));
                         }
                     } elseif (isset($map[self::KEY_TYPE]) && $map[self::KEY_TYPE] == self::TYPE_HEX) {
-                        $encoder->content(Horde_String::upper(bin2hex($this->$map[self::KEY_ATTRIBUTE])));
+                        $encoder->content(Horde_String::upper(bin2hex($this->{$map[self::KEY_ATTRIBUTE]})));
                     } elseif (isset($map[self::KEY_TYPE]) && $map[self::KEY_TYPE] == self::TYPE_MAPI_STREAM) {
-                        $encoder->content($this->$map[self::KEY_ATTRIBUTE]);
+                        $encoder->content($this->{$map[self::KEY_ATTRIBUTE]});
                     } else {
                         $encoder->content(
-                            $this->_checkEncoding($this->$map[self::KEY_ATTRIBUTE], $tag));
+                            $this->_checkEncoding($this->{$map[self::KEY_ATTRIBUTE]}, $tag));
                     }
                     $encoder->endTag();
                 }
@@ -483,8 +508,9 @@ class Horde_ActiveSync_Message_Base
         if (is_resource($data)) {
             stream_filter_register('horde_null', 'Horde_Stream_Filter_Null');
             stream_filter_register('horde_eol', 'Horde_Stream_Filter_Eol');
-            $filter_null = stream_filter_prepend($data, 'horde_null', STREAM_FILTER_READ);
-            $filter_eol = stream_filter_prepend($data, 'horde_eol', STREAM_FILTER_READ);
+            $this->_streamFilters[] = stream_filter_prepend($data, 'horde_null', STREAM_FILTER_READ);
+            $this->_streamFilters[] = stream_filter_prepend($data, 'horde_eol', STREAM_FILTER_READ);
+
         }
         return $data;
     }
@@ -547,15 +573,17 @@ class Horde_ActiveSync_Message_Base
      *
      * @param string $ts  The timestamp
      *
-     * @return Horde_Date  The Horde_Date
+     * @return Horde_Date|boolean  The Horde_Date or false if unable to decode.
      */
     protected function _parseDate($ts)
     {
         if (preg_match("/(\d{4})[^0-9]*(\d{2})[^0-9]*(\d{2})(T(\d{2})[^0-9]*(\d{2})[^0-9]*(\d{2})(.\d+)?Z){0,1}$/", $ts, $matches)) {
-            return new Horde_Date($ts);
+            try {
+                return new Horde_Date($ts);
+            } catch (Horde_Date_Exception $e) {}
         }
 
-        throw new Horde_ActiveSync_Exception('Invalid date format');
+        return false;
     }
 
     /**
@@ -565,7 +593,7 @@ class Horde_ActiveSync_Message_Base
      *
      * @return string  The binary data
      */
-    static private function hex2bin($data)
+    private static function hex2bin($data)
     {
         $len = strlen($data);
         $newdata = '';
