@@ -18,7 +18,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -30,7 +30,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  * @internal
@@ -59,7 +59,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
     const MAX_HEARTBEAT      = 3540;
 
     /**
-     * Collection of all collection arrays for the current SYNC request.
+     * Collections manager.
      *
      * @var Horde_ActiveSync_Collections
      */
@@ -129,19 +129,11 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     $this->_handleGlobalSyncError();
                     return true;
                 } else {
-                    $this->_collections->loadCollectionsFromCache();
-                    $csk = $this->_collections->confirmed_synckeys;
-                    if (count($csk) > 0) {
-                        $this->_logger->warn(sprintf(
-                            '[%s] Unconfirmed synckeys, but handling a short request. Request full SYNC.',
-                            $this->_procid));
+                    if (!$this->_collections->initEmptySync()) {
                         $this->_statusCode = self::STATUS_REQUEST_INCOMPLETE;
                         $this->_handleGlobalSyncError();
                         return true;
                     }
-                    $this->_collections->shortSyncRequest = true;
-                    $this->_collections->hangingSync = true;
-                    $this->_collections->save(true);
                 }
             } else {
                 $this->_statusCode = self::STATUS_REQUEST_INCOMPLETE;
@@ -214,9 +206,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 }
             }
 
-            // Preparation for EAS >= 12.1
             if ($this->_device->version >= Horde_ActiveSync::VERSION_TWELVEONE) {
-
                 // These are not allowed in the same request.
                 if ($this->_collections->hbinterval !== false &&
                     $this->_collections->wait !== false) {
@@ -229,10 +219,13 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     return true;
                 }
 
-                // Fill in missing information from cache.
+                // Fill in missing sticky data from cache.
                 $this->_collections->validateFromCache();
             }
 
+            // Ensure we have OPTIONS values.
+            $this->_collections->ensureOptions();
+
             // Full or partial sync request?
             if ($partial === true) {
                 $this->_logger->info(sprintf(
@@ -279,13 +272,14 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 $this->_procid));
         }
 
+        $pingSettings = $this->_driver->getHeartbeatConfig();
+
         // If this is >= 12.1, see if we want a looping SYNC.
         if ($this->_collections->canDoLoopingSync() &&
             $this->_device->version >= Horde_ActiveSync::VERSION_TWELVEONE &&
             $this->_statusCode == self::STATUS_SUCCESS) {
 
             // Calculate the heartbeat
-            $pingSettings = $this->_driver->getHeartbeatConfig();
             if (!$heartbeat = $this->_collections->getHeartbeat()) {
                 $heartbeat = !empty($pingSettings['heartbeatdefault'])
                     ? $pingSettings['heartbeatdefault']
@@ -331,7 +325,11 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             return true;
         }
 
-        // Start output to PIM
+        $this->_logger->info(sprintf(
+            '[%s] Completed parsing incoming request. Peak memory usage: %d.',
+            $this->_procid, memory_get_peak_usage(true)));
+
+        // Start output to client
         $this->_encoder->startWBXML();
         $this->_encoder->startTag(Horde_ActiveSync::SYNC_SYNCHRONIZE);
         $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
@@ -344,10 +342,16 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             $this->_encoder);
 
         $cnt_global = 0;
+        $over_window = false;
         foreach ($this->_collections as $id => $collection) {
             $statusCode = self::STATUS_SUCCESS;
             $changecount = 0;
-            $cnt_collection = 0;
+
+            if ($over_window || $cnt_global > $this->_collections->getDefaultWindowSize()) {
+                $this->_sendOverWindowResponse($collection);
+                continue;
+            }
+
             try {
                 $this->_collections->initCollectionState($collection);
             } catch (Horde_ActiveSync_Exception_StateGone $e) {
@@ -426,7 +430,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             // any changes (incoming or outgoing), if this is during the
             // initial sync pairing of the collection, or if we received a
             // SYNC due to changes found during a PING (since those changes
-            // may be changes to items that never even made it to the PIM in
+            // may be changes to items that never even made it to the client in
             // the first place (See Bug: 12075).
             if ($statusCode == self::STATUS_SUCCESS &&
                 (!empty($collection['importedchanges']) ||
@@ -474,33 +478,74 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             $this->_encoder->content($statusCode);
             $this->_encoder->endTag();
 
-            // Check the mimesupport because we need it for advanced emails
-            if (!isset($collection['mimesupport'])) {
-                $collection['mimesupport'] = 0;
-            }
-
-            $ensure_sent = array();
             if ($statusCode == self::STATUS_SUCCESS) {
+
+                // Send server changes to client
+                if ($statusCode == self::STATUS_SUCCESS &&
+                    empty($forceChanges) &&
+                    (!empty($collection['getchanges']) ||
+                     (!isset($collection['getchanges']) && !empty($collection['synckey'])))) {
+
+                    $max_windowsize = !empty($pingSettings['maximumwindowsize'])
+                        ? min($collection['windowsize'], $pingSettings['maximumwindowsize'])
+                        : $collection['windowsize'];
+
+                    if ((!empty($changecount) && $changecount > $max_windowsize) ||
+                        ($cnt_global + $changecount > $this->_collections->getDefaultWindowSize())) {
+                        $this->_logger->info(sprintf(
+                            '[%s] Sending MOREAVAILABLE. WINDOWSIZE = %d, $changecount = %d, $cnt_global = %d',
+                            $this->_procid, $max_windowsize, $changecount, $cnt_global));
+                        $this->_encoder->startTag(Horde_ActiveSync::SYNC_MOREAVAILABLE, false, true);
+                        $over_window = ($cnt_global + $changecount > $this->_collections->getDefaultWindowSize());
+                    }
+
+                    if (!empty($changecount)) {
+                        $exporter->setChanges($this->_collections->getCollectionChanges(false), $collection);
+                        $this->_encoder->startTag(Horde_ActiveSync::SYNC_COMMANDS);
+                        $cnt_collection = 0;
+                        while ($cnt_collection < $max_windowsize &&
+                               $cnt_global < $this->_collections->getDefaultWindowSize() &&
+                               $progress = $exporter->sendNextChange()) {
+
+                            $this->_logger->info(sprintf(
+                                '[%s] Peak memory usage after message: %d',
+                                $this->_procid, memory_get_peak_usage(true)));
+                            if ($progress === true) {
+                                ++$cnt_collection;
+                                ++$cnt_global;
+                            }
+                        }
+                        $this->_encoder->endTag();
+                    }
+                }
+
                 if (!empty($collection['clientids']) || !empty($collection['fetchids'])
                     || !empty($collection['missing']) || !empty($collection['importfailures'])) {
 
                     $this->_encoder->startTag(Horde_ActiveSync::SYNC_REPLIES);
 
-                    // Output any errors from missing messages in REMOVE requests.
-                    if (!empty($collection['missing'])) {
-                        foreach ($collection['missing'] as $uid) {
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_REMOVE);
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_CLIENTENTRYID);
-                            $this->_encoder->content($uid);
+                    // Output any SYNC_MODIFY failures
+                    if (!empty($collection['importfailures'])) {
+                        foreach ($collection['importfailures'] as $id => $reason) {
+                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_MODIFY);
+
+                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_FOLDERTYPE);
+                            $this->_encoder->content($collection['class']);
                             $this->_encoder->endTag();
+
+                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_SERVERENTRYID);
+                            $this->_encoder->content($id);
+                            $this->_encoder->endTag();
+
                             $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
-                            $this->_encoder->content(self::STATUS_NOTFOUND);
+                            $this->_encoder->content($reason);
                             $this->_encoder->endTag();
+
                             $this->_encoder->endTag();
                         }
                     }
 
-                    // Output server IDs for new items we received and added from PIM
+                    // Output server IDs for new items we received and added from client
                     if (!empty($collection['clientids'])) {
                         foreach ($collection['clientids'] as $clientid => $serverid) {
                             if ($serverid) {
@@ -531,25 +576,18 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                         }
                     }
 
-                    // Output any SYNC_MODIFY failures
-                    if (!empty($collection['importfailures'])) {
-                        foreach ($collection['importfailures'] as $id => $reason) {
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_MODIFY);
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_SERVERENTRYID);
-                            $this->_encoder->content($id);
+                    // Output any errors from missing messages in REMOVE requests.
+                    if (!empty($collection['missing'])) {
+                        foreach ($collection['missing'] as $uid) {
+                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_REMOVE);
+                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_CLIENTENTRYID);
+                            $this->_encoder->content($uid);
                             $this->_encoder->endTag();
                             $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
-                            $this->_encoder->content($reason);
+                            $this->_encoder->content(self::STATUS_NOTFOUND);
                             $this->_encoder->endTag();
                             $this->_encoder->endTag();
-                            // If we have a conflict, ensure we send the new server
-                            // data in the response, if possible. Some android
-                            // clients require this, or never accept the response.
-                            if ($reason == self::STATUS_CONFLICT) {
-                                $ensure_sent[] = $id;
-                            }
                         }
-
                     }
 
                     if (!empty($collection['fetchids'])) {
@@ -589,39 +627,6 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     $this->_encoder->endTag();
                 }
 
-                // Send server changes to PIM
-                if ($statusCode == self::STATUS_SUCCESS &&
-                    empty($forceChanges) &&
-                    (!empty($collection['getchanges']) ||
-                     (!isset($collection['getchanges']) && !empty($collection['synckey'])))) {
-
-                    if ((!empty($changecount) && $changecount > $collection['windowsize']) ||
-                        ($cnt_global + $changecount > $this->_collections->getDefaultWindowSize())) {
-
-                        $this->_logger->info(sprintf(
-                            '[%s] Sending MOREAVAILABLE. $changecount = %d, $cnt_global = %d',
-                            $this->_procid, $changecount, $cnt_global));
-                        $this->_encoder->startTag(Horde_ActiveSync::SYNC_MOREAVAILABLE, false, true);
-                    }
-
-                    if (!empty($changecount)) {
-                        $exporter->setChanges($this->_collections->getCollectionChanges(false, $ensure_sent), $collection);
-                        $this->_encoder->startTag(Horde_ActiveSync::SYNC_COMMANDS);
-                        $cnt_collection = 0;
-                        while ($cnt_collection < $collection['windowsize'] &&
-                               $cnt_global < $this->_collections->getDefaultWindowSize() &&
-                               $progress = $exporter->sendNextChange()) {
-
-                            if ($progress === true) {
-                                ++$cnt_collection;
-                                ++$cnt_global;
-                            }
-                        }
-
-                        $this->_encoder->endTag();
-                    }
-                }
-
                 // Save the sync state for the next time
                 if (!empty($collection['newsynckey'])) {
                     if (!empty($sync) || !empty($importer) || $collection['synckey'] == 0)  {
@@ -646,6 +651,9 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 }
             }
             $this->_encoder->endTag();
+            $this->_logger->info(sprintf(
+                '[%s] Collection output peak memory usage: %d',
+                $this->_procid, memory_get_peak_usage(true)));
         }
 
         $this->_encoder->endTag();
@@ -668,6 +676,39 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
         return true;
     }
 
+
+    protected function _sendOverWindowResponse($collection)
+    {
+        $this->_logger->info(sprintf(
+            '[%s] Over window maximum, skip polling for this request.',
+            $this->_procid));
+
+        $this->_encoder->startTag(Horde_ActiveSync::SYNC_FOLDER);
+
+        // Not sent in > 12.0
+        if ($this->_device->version <= Horde_ActiveSync::VERSION_TWELVE) {
+            $this->_encoder->startTag(Horde_ActiveSync::SYNC_FOLDERTYPE);
+            $this->_encoder->content($collection['class']);
+            $this->_encoder->endTag();
+        }
+
+        $this->_encoder->startTag(Horde_ActiveSync::SYNC_SYNCKEY);
+        $this->_encoder->content($collection['synckey']);
+        $this->_encoder->endTag();
+
+        $this->_encoder->startTag(Horde_ActiveSync::SYNC_FOLDERID);
+        $this->_encoder->content($collection['id']);
+        $this->_encoder->endTag();
+
+        $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
+        $this->_encoder->content(self::STATUS_SUCCESS); //??
+        $this->_encoder->endTag();
+        $this->_encoder->startTag(Horde_ActiveSync::SYNC_MOREAVAILABLE, false, true);
+
+        $this->_encoder->endTag();
+        return;
+    }
+
     /**
      * Helper method for parsing incoming SYNC_FOLDERS nodes.
      *
@@ -709,6 +750,15 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
 
                 case Horde_ActiveSync::SYNC_FOLDERID:
                     $collection['id'] = $this->_decoder->getElementContent();
+                    if ($collection['id'] === false) {
+                        // Log this case explicitly since we can't send back
+                        // a protocol error status (the response requires a
+                        // collection id and we obviously don't have one).
+                        $this->_logger->err(sprintf(
+                            '[%s] PROTOCOL ERROR. Client sent an empty SYNC_FOLDERID value.',
+                            $this->_procid));
+                        throw new Horde_ActiveSync_Exception('Protocol error');
+                    }
                     if (!$this->_decoder->getElementEndTag()) {
                         throw new Horde_ActiveSync_Exception('Protocol error');
                     }
@@ -812,6 +862,14 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 return false;
             }
 
+            if (isset($collection['filtertype']) &&
+                !$this->_collections->checkFilterType($collection['id'], $collection['filtertype'])) {
+
+                $this->_logger->info(sprintf(
+                    '[%s] Found updated filtertype, will force a SOFTDELETE.', $this->_procid));
+                $collection['forcerefresh'] = true;
+            }
+
             try {
                 $this->_collections->addCollection($collection);
             } catch (Horde_ActiveSync_Exception_StateGone $e) {
@@ -837,12 +895,6 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             return false;
         }
 
-        if (isset($collection['filtertype']) &&
-            !$this->_collections->checkFilterType($collection['id'], $collection['filtertype'])) {
-            $this->_statusCode = self::STATUS_KEYMISM;
-            $this->_handleError($collection);
-        }
-
         return true;
     }
 
@@ -956,7 +1008,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 $clientid = false;
             }
 
-            // Create Message object from messages passed from PIM.
+            // Create Message object from messages passed from client.
             // Only passed during SYNC_ADD or SYNC_MODIFY
             if (($element[Horde_ActiveSync_Wbxml::EN_TAG] == Horde_ActiveSync::SYNC_ADD ||
                 $element[Horde_ActiveSync_Wbxml::EN_TAG] == Horde_ActiveSync::SYNC_MODIFY) &&
@@ -1131,9 +1183,10 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 $this->_mimeSupport($options);
             }
 
+            // SYNC_MIMETRUNCATION is used when no SYNC_BODYPREFS element is sent.
             if ($this->_decoder->getElementStartTag(Horde_ActiveSync::SYNC_MIMETRUNCATION)) {
                 $haveElement = true;
-                $options['mimetruncation'] = $this->_decoder->getElementContent();
+                $options['mimetruncation'] = Horde_ActiveSync::getMIMETruncSize($this->_decoder->getElementContent());
                 if (!$this->_decoder->getElementEndTag()) {
                     $this->_statusCode = self::STATUS_PROTERROR;
                     $this->_handleError($collection);
@@ -1141,9 +1194,11 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 }
             }
 
+            // SYNC_TRUNCATION only applies to the body of non-email collections
+            // or the BODY element of an Email in EAS 2.5.
             if ($this->_decoder->getElementStartTag(Horde_ActiveSync::SYNC_TRUNCATION)) {
                 $haveElement = true;
-                $options['truncation'] = $this->_decoder->getElementContent();
+                $options['truncation'] = Horde_ActiveSync::getTruncSize($this->_decoder->getElementContent());
                 if (!$this->_decoder->getElementEndTag()) {
                     $this->_statusCode = self::STATUS_PROTERROR;
                     $this->_handleError($collection);
