@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -20,19 +20,18 @@
   *            Version 2, the distribution of the Horde_ActiveSync module in or
   *            to the United States of America is excluded from the scope of this
   *            license.
-  * @copyright 2010-2014 Horde LLC (http://www.horde.org)
+  * @copyright 2010-2015 Horde LLC (http://www.horde.org)
   * @author    Michael J Rubinsky <mrubinsk@horde.org>
   * @package   ActiveSync
   */
 class Horde_ActiveSync_Rfc822
 {
-
     /**
      * The memory limit for use with the PHP temp stream.
      *
      * @var integer
      */
-    static public $memoryLimit = 2097152;
+    public static $memoryLimit = 2097152;
 
     /**
      * Position of end of headers.
@@ -112,7 +111,8 @@ class Horde_ActiveSync_Rfc822
     }
 
     /**
-     * Replace the MIME part of the message sent from the client.
+     * Replace the MIME part of the message sent from the client. Headers from
+     * the original message are always used.
      *
      * @param  Horde_Mime_Part $part  The new MIME part.
      * @since 2.19.0
@@ -123,21 +123,20 @@ class Horde_ActiveSync_Rfc822
             'stream' => true,
             'headers' => false)
         );
+        $mime_stream = new Horde_Stream_Existing(array('stream' => $mime_stream));
 
-        if (!empty($this->_header_text)) {
-            $hdr = $this->_header_text;
-        } else {
-            $this->_stream->rewind();
-            $hdr = $this->_stream->substring(0, $this->_hdr_pos);
-        }
-        $new_stream = Horde_Stream_Wrapper_Combine::getStream(array($hdr, $mime_stream));
-        $this->_parseStream(new Horde_Stream_Existing(array('stream' => $new_stream)));
+        // Since we are still using the headers sent from the device, we can
+        // simply zero out the position members etc...
+        $this->_hdr_pos = 0;
+        $this->_stream = $mime_stream;
+        $mime_stream->rewind();
     }
 
     /**
      * Return the raw message data.
      *
      * @return stream resource
+     * @todo Rename to make it clear this returns a stream.
      */
     public function getString()
     {
@@ -162,6 +161,7 @@ class Horde_ActiveSync_Rfc822
             $this->_stream->rewind();
             $hdr_text = $this->_stream->substring(0, $this->_hdr_pos);
         }
+
         return Horde_Mime_Headers::parseHeaders($hdr_text);
     }
 
@@ -183,6 +183,11 @@ class Horde_ActiveSync_Rfc822
             $headers->addUserAgentHeader();
             $updated = true;
         }
+        if (!$headers->getValue('Date')) {
+            $d = new Horde_Date();
+            $headers->addHeaderOb(Horde_Mime_Headers_Date::create());
+            $updated = true;
+        }
         if ($updated) {
             $this->_header_text = $headers->toString(array('charset' => 'UTF-8'));
         }
@@ -224,21 +229,17 @@ class Horde_ActiveSync_Rfc822
      */
     protected function _findHeader()
     {
-        $i = 0;
-        while (!$this->_stream->eof()) {
-            $data = $this->_stream->substring(0, 8192);
-            $hdr_pos = strpos($data, "\r\n\r\n");
-            if ($hdr_pos !== false) {
-                return array($hdr_pos + ($i * 8192), 4);
-            }
-            $hdr_pos = strpos($data, "\n\n");
-            if ($hdr_pos !== false) {
-                return array($hdr_pos + ($i * 8192), 2);
-            }
-            $i++;
+        // Look for the EOL that is found first in the message. Some clients
+        // are sending mixed EOL in S/MIME signed messages. This still doesn't
+        // fix "Nine" currently, as they first send \r\n, but use \n\n to
+        // separate the headers.
+        // See: https://ninefolders.plan.io/track/10606/1dcfed
+        switch ($this->_stream->getEOL()) {
+        case "\n":
+            return array($this->_stream->search("\n\n"), 2);
+        case "\r\n":
+            return array($this->_stream->search("\r\n\r\n"), 4);
         }
-        $this->_stream->end();
-        return array($this->_stream->pos(), 0);
     }
 
 }
