@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -19,7 +19,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -80,7 +80,7 @@ abstract class Horde_ActiveSync_State_Base
      * Keys include:
      *   - class:       The collection class Contacts, Calendar etc...
      *   - synckey:     The current synckey
-     *   - newsynckey:  The new synckey sent back to the PIM
+     *   - newsynckey:  The new synckey sent back to the client
      *   - id:          Server folder id
      *   - filtertype:  Filter
      *   - conflict:    Conflicts
@@ -105,7 +105,7 @@ abstract class Horde_ActiveSync_State_Base
     protected $_deviceInfo;
 
     /**
-     * Local cache for changes to *send* to PIM
+     * Local cache for changes to *send* to client.
      * (Will remain null until getChanges() is called)
      *
      * @var array
@@ -135,21 +135,13 @@ abstract class Horde_ActiveSync_State_Base
     {
         $this->_params = $params;
         if (empty($params['logger'])) {
-            $this->_logger = new Horde_Support_Stub();
+            $this->_logger =  new Horde_Log_Logger(new Horde_Log_Handler_Null());
         } else {
             $this->_logger = $params['logger'];
         }
         $this->_procid = getmypid();
     }
 
-    /**
-     * Destructor
-     */
-    public function __destruct()
-    {
-        unset($this->_backend);
-    }
-
     /**
      * Update the $oldKey syncState to $newKey.
      *
@@ -257,10 +249,10 @@ abstract class Horde_ActiveSync_State_Base
 
     /**
      * Determines if the server version of the message represented by $stat
-     * conflicts with the PIM version of the message.  For this driver, this is
+     * conflicts with the client version of the message.  For this driver, this is
      * true whenever $lastSyncTime is older then $stat['mod']. Method is only
      * called from the Importer during an import of a non-new change from the
-     * PIM.
+     * client.
      *
      * @param array $stat   A message stat array
      * @param string $type  The type of change (change, delete, add)
@@ -429,7 +421,8 @@ abstract class Horde_ActiveSync_State_Base
                 $cutoffdate,
                 !empty($options['ping']),
                 $this->_folder->haveInitialSync,
-                !empty($options['maxitems']) ? $options['maxitems'] : 100
+                !empty($options['maxitems']) ? $options['maxitems'] : 100,
+                !empty($this->_collection['forcerefresh'])
             );
 
             // Only update the folderstate if we are not PINGing.
@@ -448,19 +441,29 @@ abstract class Horde_ActiveSync_State_Base
             $this->_changes = array();
             if (count($changes) && $this->_havePIMChanges()) {
                 $this->_logger->info(sprintf(
-                    '[%s] Checking for PIM initiated changes.',
+                    '[%s] Checking for client initiated changes.',
                     $this->_procid));
 
                 switch ($this->_collection['class']) {
                 case Horde_ActiveSync::CLASS_EMAIL:
+                    // @todo Fix me with a changes object that transparently
+                    // deals with different data structure for initial sync.
+                    // ...or come up with better solution for dealing with
+                    // memory usage.
+                    if (!empty($changes) && !is_array($changes[0])) {
+                        $this->_changes = $changes;
+                        break;
+                    }
+
                     $mailmap = $this->_getMailMapChanges($changes);
                     $flag_map = array(
                         Horde_ActiveSync::CHANGE_TYPE_FLAGS =>  'flag change',
                         Horde_ActiveSync::CHANGE_TYPE_DELETE => 'deletion',
                         Horde_ActiveSync::CHANGE_TYPE_CHANGE => 'move'
                     );
-                    foreach ($changes as $change) {
-                        if (!empty($mailmap[$change['id']][$change['type']])) {
+                    $cnt = count($changes);
+                    for ($i = 0; $i < $cnt; $i++) {
+                        if (!empty($mailmap[$changes[$i]['id']][$changes[$i]['type']])) {
                             // @todo For 3.0, create a Changes and
                             // ChangeFilter classes to abstract out a bunch of
                             // this stuff. (Needs BC breaking changes in
@@ -470,52 +473,53 @@ abstract class Horde_ActiveSync_State_Base
                             // email during MOVEITEMS requests (instead it
                             // reassigns the existing email the new UID). Don't
                             // send the ADD command for these changes.
-                            if ($change['type'] == Horde_ActiveSync::CHANGE_TYPE_CHANGE &&
-                                $change['flags'] == Horde_ActiveSync::FLAG_NEWMESSAGE &&
+                            if ($changes[$i]['type'] == Horde_ActiveSync::CHANGE_TYPE_CHANGE &&
+                                $changes[$i]['flags'] == Horde_ActiveSync::FLAG_NEWMESSAGE &&
                                 $this->_deviceInfo->deviceType != 'WindowsOutlook15') {
-                                $this->_changes[] = $change;
+                                $this->_changes[] = $changes[$i];
                                 continue;
                             }
                             $this->_logger->info(sprintf(
-                                '[%s] Ignoring PIM initiated %s for %s',
+                                '[%s] Ignoring client initiated %s for %s',
                                 $this->_procid,
-                                $flag_map[$change['type']],
-                                $change['id']));
-                            $change['ignore'] = true;
+                                $flag_map[$changes[$i]['type']],
+                                $changes[$i]['id']));
+                            $changes[$i]['ignore'] = true;
                         }
-                        $this->_changes[] = $change;
+                        $this->_changes[] = $changes[$i];
                     }
                     break;
 
                 default:
-                    $pim_timestamps = $this->_getPIMChangeTS($changes);
-                    foreach ($changes as $change) {
-                        if (empty($pim_timestamps[$change['id']])) {
-                            $this->_changes[] = $change;
+                    $client_timestamps = $this->_getPIMChangeTS($changes);
+                    $cnt = count($changes);
+                    for ($i = 0; $i < $cnt; $i++) {
+                        if (empty($client_timestamps[$changes[$i]['id']])) {
+                            $this->_changes[] = $changes[$i];
                             continue;
                         }
-                        if ($change['type'] == Horde_ActiveSync::CHANGE_TYPE_DELETE) {
+                        if ($changes[$i]['type'] == Horde_ActiveSync::CHANGE_TYPE_DELETE) {
                             // If we have a delete, don't bother stating the message,
                             // the entry should already be deleted on the client.
                             $stat['mod'] = 0;
                         } else {
                             // stat only returns MODIFY times, not deletion times,
                             // so will return (int)0 for ADD or DELETE.
-                            $stat = $this->_backend->statMessage($this->_folder->serverid(), $change['id']);
+                            $stat = $this->_backend->statMessage($this->_folder->serverid(), $changes[$i]['id']);
                         }
-                        if ($pim_timestamps[$change['id']] >= $stat['mod']) {
+                        if ($client_timestamps[$changes[$i]['id']] >= $stat['mod']) {
                             $this->_logger->info(sprintf(
-                                '[%s] Ignoring PIM initiated change for %s (PIM TS: %s Stat TS: %s)',
+                                '[%s] Ignoring client initiated change for %s (client TS: %s Stat TS: %s)',
                                 $this->_procid,
-                                $change['id'], $pim_timestamps[$change['id']], $stat['mod']));
+                                $changes[$i]['id'], $client_timestamps[$changes[$i]['id']], $stat['mod']));
                         } else {
-                            $this->_changes[] = $change;
+                            $this->_changes[] = $changes[$i];
                         }
                     }
                 }
             } elseif (count($changes)) {
                 $this->_logger->info(sprintf(
-                    '[%s] No PIM changes present, returning all messages.',
+                    '[%s] No client changes present, returning all messages.',
                     $this->_procid));
                 $this->_changes = $changes;
             }
@@ -576,7 +580,7 @@ abstract class Horde_ActiveSync_State_Base
      * @return string  The new synckey
      * @throws Horde_ActiveSync_Exception
      */
-    static public function getNewSyncKey($syncKey)
+    public static function getNewSyncKey($syncKey)
     {
         if (empty($syncKey)) {
             return '{' . new Horde_Support_Uuid() . '}' . '1';
@@ -598,7 +602,7 @@ abstract class Horde_ActiveSync_State_Base
      *
      * @return mixed integer|boolean  The increment counter or false if failed.
      */
-    static public function getSyncKeyCounter($syncKey)
+    public static function getSyncKeyCounter($syncKey)
     {
        if (preg_match('/^\{([a-fA-F0-9-]+)\}([0-9]+)$/', $syncKey, $matches)) {
             $n = $matches[2];
@@ -615,7 +619,7 @@ abstract class Horde_ActiveSync_State_Base
      *
      * @return string  The UID.
      */
-    static public function getSyncKeyUid($syncKey)
+    public static function getSyncKeyUid($syncKey)
     {
        if (preg_match('/^(\{[a-fA-F0-9-]+\})([0-9]+)$/', $syncKey, $matches)) {
             return $matches[1];
@@ -629,7 +633,7 @@ abstract class Horde_ActiveSync_State_Base
     *
     * @return integer
     */
-    static protected function _getCutOffDate($restrict)
+    protected static function _getCutOffDate($restrict)
     {
         // @todo Should just pass the filtertype to the driver instead
         // of parsing it here, let the driver figure out what to do with it.
@@ -671,10 +675,10 @@ abstract class Horde_ActiveSync_State_Base
     }
 
     /**
-     * Helper function that performs the actual diff between PIM state and
+     * Helper function that performs the actual diff between client state and
      * server state FOLDERSYNC arrays.
      *
-     * @param array $old  The PIM state
+     * @param array $old  The client state
      * @param array $new  The current server state
      *
      * @return unknown_type
@@ -756,7 +760,7 @@ abstract class Horde_ActiveSync_State_Base
      * @param $b
      * @return unknown_type
      */
-    static public function RowCmp($a, $b)
+    public static function RowCmp($a, $b)
     {
         return $a['id'] < $b['id'] ? 1 : -1;
     }
@@ -862,8 +866,8 @@ abstract class Horde_ActiveSync_State_Base
      * and user.
      *
      * An extra database query for each sync, but the payoff is that we avoid
-     * having to stat every message change we send to the PIM if there are no
-     * PIM generated changes for this sync period.
+     * having to stat every message change we send to the client if there are no
+     * client generated changes for this sync period.
      *
      * @return boolean
      * @throws Horde_ActiveSync_Exception
@@ -909,11 +913,11 @@ abstract class Horde_ActiveSync_State_Base
      *
      * @param integer $origin   Flag to indicate the origin of the change:
      *    Horde_ActiveSync::CHANGE_ORIGIN_NA  - Not applicapble/not important
-     *    Horde_ActiveSync::CHANGE_ORIGIN_PIM - Change originated from PIM
+     *    Horde_ActiveSync::CHANGE_ORIGIN_PIM - Change originated from client
      *
      * @param string $user      The current sync user, only needed if change
      *                          origin is CHANGE_ORIGIN_PIM
-     * @param string $clientid  PIM clientid sent when adding a new message
+     * @param string $clientid  client clientid sent when adding a new message
      */
     abstract public function updateState(
         $type, array $change, $origin = Horde_ActiveSync::CHANGE_ORIGIN_NA,
@@ -1052,8 +1056,8 @@ abstract class Horde_ActiveSync_State_Base
     abstract public function deleteSyncCache($devid, $user);
 
     /**
-     * Check and see that we didn't already see the incoming change from the PIM.
-     * This would happen e.g., if the PIM failed to receive the server response
+     * Check and see that we didn't already see the incoming change from the client.
+     * This would happen e.g., if the client failed to receive the server response
      * after successfully importing new messages.
      *
      * @param string $id  The client id sent during message addition.
