@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -65,7 +65,7 @@
  *          - pingheartbeat:
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- * @copyright 2010-2014 Horde LLC (http://www.horde.org/)
+ * @copyright 2010-2015 Horde LLC (http://www.horde.org/)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @link      http://pear.horde.org/index.php?package=ActiveSync
  * @package   ActiveSync
@@ -397,7 +397,7 @@ class Horde_ActiveSync_State_Mongo extends Horde_ActiveSync_State_Base implement
     /**
      * Update the state to reflect changes
      *
-     * Notes: If we are importing PIM changes, need to update the syncMapTable
+     * Notes: If we are importing client changes, need to update the syncMapTable
      * so we don't mirror back the changes on next sync. If we are exporting
      * server changes, we need to track which changes have been sent (by
      * removing them from $this->_changes) so we know which items to send on the
@@ -417,11 +417,11 @@ class Horde_ActiveSync_State_Mongo extends Horde_ActiveSync_State_Base implement
      *
      * @param integer $origin   Flag to indicate the origin of the change:
      *    Horde_ActiveSync::CHANGE_ORIGIN_NA  - Not applicapble/not important
-     *    Horde_ActiveSync::CHANGE_ORIGIN_PIM - Change originated from PIM
+     *    Horde_ActiveSync::CHANGE_ORIGIN_PIM - Change originated from client
      *
      * @param string $user      The current sync user, only needed if change
      *                          origin is CHANGE_ORIGIN_PIM
-     * @param string $clientid  PIM clientid sent when adding a new message.
+     * @param string $clientid  client clientid sent when adding a new message.
      *
      * @throws  Horde_ActiveSync_Exception
      */
@@ -460,7 +460,7 @@ class Horde_ActiveSync_State_Mongo extends Horde_ActiveSync_State_Base implement
                 ? $this->getLatestSynckeyForCollection($this->_collection['id'])
                 : $this->_syncKey;
 
-            // This is an incoming change from the PIM, store it so we
+            // This is an incoming change from the client, store it so we
             // don't mirror it back to device.
             switch ($this->_collection['class']) {
             case Horde_ActiveSync::CLASS_EMAIL:
@@ -927,9 +927,6 @@ class Horde_ActiveSync_State_Mongo extends Horde_ActiveSync_State_Base implement
         if (empty($results) || empty($results['ok'])) {
             throw new Horde_ActiveSync_Exception(empty($results['errmsg']) ? 'Error running aggregation.' : $results['errmsg']);
         }
-        if (empty($results) || empty($results['ok'])) {
-            return 0;
-        }
         $results = current($results['result']);
         return $results['max'];
     }
@@ -1106,8 +1103,8 @@ class Horde_ActiveSync_State_Mongo extends Horde_ActiveSync_State_Base implement
     }
 
     /**
-     * Check and see that we didn't already see the incoming change from the PIM.
-     * This would happen e.g., if the PIM failed to receive the server response
+     * Check and see that we didn't already see the incoming change from the client.
+     * This would happen e.g., if the client failed to receive the server response
      * after successfully importing new messages.
      *
      * @param string $id  The client id sent during message addition.
@@ -1296,12 +1293,12 @@ class Horde_ActiveSync_State_Mongo extends Horde_ActiveSync_State_Base implement
 
     /**
      * Return an array of timestamps from the map table for the last
-     * PIM-initiated change for the provided uid. Used to avoid mirroring back
-     * changes to the PIM that it sent to the server.
+     * client-initiated change for the provided uid. Used to avoid mirroring back
+     * changes to the client that it sent to the server.
      *
      * @param array $changes  The changes array, containing 'id' and 'type'.
      *
-     * @return array  An array of UID -> timestamp of the last PIM-initiated
+     * @return array  An array of UID -> timestamp of the last client-initiated
      *                change for the specified uid, or null if none found.
      */
     protected function _getPIMChangeTS(array $changes)
@@ -1360,8 +1357,8 @@ class Horde_ActiveSync_State_Mongo extends Horde_ActiveSync_State_Base implement
      * and user.
      *
      * An extra database query for each sync, but the payoff is that we avoid
-     * having to stat every message change we send to the PIM if there are no
-     * PIM generated changes for this sync period.
+     * having to stat every message change we send to the client if there are no
+     * client generated changes for this sync period.
      *
      * @return boolean
      * @throws Horde_ActiveSync_Exception
@@ -1463,7 +1460,7 @@ class Horde_ActiveSync_State_Mongo extends Horde_ActiveSync_State_Base implement
 
         // Clean up all but the last 2 syncs for any given sync series, this
         // ensures that we can still respond to SYNC requests for the previous
-        // key if the PIM never received the new key in a SYNC response.
+        // key if the client never received the new key in a SYNC response.
         $js = <<<EOT
         function() {
             var p = /^\{([0-9A-Za-z-]+)\}([0-9]+)$/;
