@@ -9,7 +9,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -36,7 +36,7 @@
  *
  *    syncMapTable (horde_activesync_map):
  *        message_uid    - The server uid for the object
- *        sync_modtime   - The time the change was received from the PIM and
+ *        sync_modtime   - The time the change was received from the client and
  *                         applied to the server data store.
  *        sync_key       - The syncKey that was current at the time the change
  *                         was received.
@@ -45,7 +45,7 @@
  *
  *    syncDeviceTable (horde_activesync_device):
  *        device_id         - The unique id for this device
- *        device_type       - The device type the PIM identifies itself with
+ *        device_type       - The device type the client identifies itself with
  *        device_agent      - The user agent string sent by the device
  *        device_policykey  - The current policykey for this device
  *        device_rwstatus   - The current remote wipe status for this device
@@ -57,7 +57,7 @@
  *                           combination.
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- * @copyright 2010-2014 Horde LLC (http://www.horde.org/)
+ * @copyright 2010-2015 Horde LLC (http://www.horde.org/)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @link      http://pear.horde.org/index.php?package=ActiveSync
  * @package   ActiveSync
@@ -159,11 +159,11 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
             $this->_procid,
             $serverid,
             $uid));
-        $sql = 'SELECT sync_data FROM ' . $this->_syncStateTable . ' WHERE '
+        $sql = 'SELECT sync_key, sync_data FROM ' . $this->_syncStateTable . ' WHERE '
             . 'sync_devid = ? AND sync_user = ? AND sync_folderid = ?';
 
         try {
-            $results = $this->_db->selectValues($sql,
+            $results = $this->_db->selectAll($sql,
                 array($this->_deviceInfo->id, $this->_deviceInfo->user, $uid));
         } catch (Horde_Db_Exception $e) {
             $this->_logger->err(sprintf(
@@ -173,16 +173,36 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
             );
             throw new Horde_ActiveSync_Exception($e);
         }
-        $update = 'UPDATE ' . $this->_syncStateTable . ' SET sync_data = ? WHERE '
-            . 'sync_devid = ? AND sync_user = ? AND sync_folderid = ?';
 
-        foreach ($results as $folder) {
-            $folder = unserialize($folder);
+        try {
+            $columns = $this->_db->columns($this->_syncStateTable);
+        } catch (Horde_Db_Exception $e) {
+            $this->_logger->err(sprintf(
+                '[%s] %s',
+                $this->_procid,
+                $e->getMessage())
+            );
+            throw new Horde_ActiveSync_Exception($e);
+        }
+
+
+        $update = 'UPDATE ' . $this->_syncStateTable . ' SET sync_data = ? WHERE '
+            . 'sync_devid = ? AND sync_user = ? AND sync_folderid = ? AND sync_key = ?';
+
+        foreach ($results as $result) {
+            $folder = unserialize($columns['sync_data']->binaryToString($result['sync_data']));
             $folder->setServerId($serverid);
             $folder = serialize($folder);
             try {
                 $this->_db->update($update,
-                    array($folder, $this->_deviceInfo->id, $this->_deviceInfo->user, $uid));
+                    array(
+                        new Horde_Db_Value_Binary($folder),
+                        $this->_deviceInfo->id,
+                        $this->_deviceInfo->user,
+                        $uid,
+                        $result['sync_key']
+                    )
+                );
             } catch (Horde_Db_Exception $e) {
                 $this->_logger->err(sprintf(
                     '[%s] %s',
@@ -352,7 +372,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
     /**
      * Update the state to reflect changes
      *
-     * Notes: If we are importing PIM changes, need to update the syncMapTable
+     * Notes: If we are importing client changes, need to update the syncMapTable
      * so we don't mirror back the changes on next sync. If we are exporting
      * server changes, we need to track which changes have been sent (by
      * removing them from $this->_changes) so we know which items to send on the
@@ -372,11 +392,11 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      *
      * @param integer $origin   Flag to indicate the origin of the change:
      *    Horde_ActiveSync::CHANGE_ORIGIN_NA  - Not applicapble/not important
-     *    Horde_ActiveSync::CHANGE_ORIGIN_PIM - Change originated from PIM
+     *    Horde_ActiveSync::CHANGE_ORIGIN_PIM - Change originated from client
      *
      * @param string $user      The current sync user, only needed if change
      *                          origin is CHANGE_ORIGIN_PIM
-     * @param string $clientid  PIM clientid sent when adding a new message
+     * @param string $clientid  client clientid sent when adding a new message
      *
      * @todo This method needs some cleanup, abstraction.
      */
@@ -413,7 +433,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                 ? $this->getLatestSynckeyForCollection($this->_collection['id'])
                 : $this->_syncKey;
 
-            // This is an incoming change from the PIM, store it so we
+            // This is an incoming change from the client, store it so we
             // don't mirror it back to device.
             switch ($this->_collection['class']) {
             case Horde_ActiveSync::CLASS_EMAIL:
@@ -487,8 +507,10 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
             // changes so we need to track which ones are sent since not all
             // may be sent. We need to store the leftovers for sending next
             // request.
+            // @todo FIX BC HACK for differing data structures when sending
+            // initial change set.
             foreach ($this->_changes as $key => $value) {
-                if ($value['id'] == $change['id']) {
+                if ((is_array($value) && $value['id'] == $change['id']) || $value == $change['id']) {
                     if ($this->_type == Horde_ActiveSync::REQUEST_TYPE_FOLDERSYNC) {
                         foreach ($this->_folder as $fi => $state) {
                             if ($state['id'] == $value['id']) {
@@ -988,8 +1010,8 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
     }
 
     /**
-     * Check and see that we didn't already see the incoming change from the PIM.
-     * This would happen e.g., if the PIM failed to receive the server response
+     * Check and see that we didn't already see the incoming change from the client.
+     * This would happen e.g., if the client failed to receive the server response
      * after successfully importing new messages.
      *
      * @param string $id  The client id sent during message addition.
@@ -1156,13 +1178,13 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
 
     /**
      * Return an array of timestamps from the map table for the last
-     * PIM-initiated change for the provided uid. Used to avoid mirroring back
-     * changes to the PIM that it sent to the server.
+     * client-initiated change for the provided uid. Used to avoid mirroring back
+     * changes to the client that it sent to the server.
      *
      * @param array $changes  The changes array, each entry a hash containing
      *                        'id' and 'type' keys.
      *
-     * @return array  An array of UID -> timestamp of the last PIM-initiated
+     * @return array  An array of UID -> timestamp of the last client-initiated
      *                change for the specified UIDs, or null if none found.
      */
     protected function _getPIMChangeTS(array $changes)
@@ -1201,8 +1223,8 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      * and user.
      *
      * An extra database query for each sync, but the payoff is that we avoid
-     * having to stat every message change we send to the PIM if there are no
-     * PIM generated changes for this sync period.
+     * having to stat every message change we send to the client if there are no
+     * client generated changes for this sync period.
      *
      * @return boolean
      * @throws Horde_ActiveSync_Exception
@@ -1301,7 +1323,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
 
         // Clean up all but the last 2 syncs for any given sync series, this
         // ensures that we can still respond to SYNC requests for the previous
-        // key if the PIM never received the new key in a SYNC response.
+        // key if the client never received the new key in a SYNC response.
         $sql = 'SELECT sync_key FROM ' . $this->_syncStateTable
             . ' WHERE sync_devid = ? AND sync_folderid = ?';
         $values = array(
