@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -20,7 +20,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  * @internal  Not intended for use outside of the Horde_ActiveSync library.
@@ -96,6 +96,7 @@ class Horde_ActiveSync_SyncCache
      * @param Horde_ActiveSync_State_Base $state  The state driver
      * @param string $devid                       The device id
      * @param string $user                        The username
+     * @param Horde_Log_Logger $logger            The logger object
      *
      * @return Horde_ActiveSync_SyncCache
      */
@@ -111,6 +112,15 @@ class Horde_ActiveSync_SyncCache
         $this->_logger = $logger;
         $this->loadCacheFromStorage();
         $this->_procid = getmypid();
+
+        $this->_logger = empty($logger)
+            ? new Horde_Log_Logger(new Horde_Log_Handler_Null())
+            : $logger;
+
+        $this->_logger->info(sprintf(
+            '[%s] Creating new Horde_ActiveSync_SyncCache.',
+            $this->_procid)
+        );
     }
 
     public function __get($property)
@@ -437,7 +447,7 @@ class Horde_ActiveSync_SyncCache
      */
     public function removeCollection($id, $purge = true)
     {
-        if (!$purge) {
+        if ($purge) {
             $this->_logger->info(sprintf(
                 '[%s] Removing collection %s from SyncCache.',
                 $this->_procid, $id)
@@ -462,6 +472,19 @@ class Horde_ActiveSync_SyncCache
         $this->_markCollectionsDirty($id);
     }
 
+    /**
+     * Update the filtertype for the specified collection.
+     *
+     * @param string $id           The collection id.
+     * @param integer $filtertype  The updated filtertype.
+     * @since 2.26.0
+     */
+    public function updateFiltertype($id, $filtertype)
+    {
+        $this->_data['collections'][$id]['filtertype'] = $filtertype;
+        $this->_markCollectionsDirty($id);
+    }
+
     /**
      * Clear all synckeys from the known collections.
      *
@@ -497,6 +520,7 @@ class Horde_ActiveSync_SyncCache
     public function removeConfirmedKey($key)
     {
         unset($this->_data['confirmed_synckeys'][$key]);
+        $this->_dirty['confirmed_synckeys'] = true;
     }
 
     /**
@@ -566,6 +590,9 @@ class Horde_ActiveSync_SyncCache
                 $this->_data['collections'][$collection['id']]['bodyprefs'] = $collection['bodyprefs'];
                 $this->_markCollectionsDirty($collection['id']);
             }
+            if (isset($collection['bodypartprefs'])) {
+                $this->_data['collections'][$collection['id']]['bodypartprefs'] = $collection['bodypartprefs'];
+            }
             if (isset($collection['pingable'])) {
                 $this->_data['collections'][$collection['id']]['pingable'] = $collection['pingable'];
                 $this->_markCollectionsDirty($collection['id']);
