@@ -7,14 +7,14 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
 /**
  * Utility functions for dealing with Microsoft ActiveSync's Timezone format.
  *
- * Copyright 2009-2014 Horde LLC (http://www.horde.org/)
+ * Copyright 2009-2015 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -27,7 +27,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  * @deprecated To be removed for Horde 6. Use Horde_Mapi_Timezone::.
@@ -77,9 +77,9 @@ class Horde_ActiveSync_Timezone
      *
      * @return array  Hash of offset information
      */
-    static public function getOffsetsFromSyncTZ($data)
+    public static function getOffsetsFromSyncTZ($data)
     {
-        if (PHP_MINOR_VERSION >= 5) {
+        if (version_compare(PHP_VERSION, '5.5', '>=')) {
             $format = 'lbias/Z64stdname/vstdyear/vstdmonth/vstdday/vstdweek/vstdhour/vstdminute/vstdsecond/vstdmillis/'
                 . 'lstdbias/Z64dstname/vdstyear/vdstmonth/vdstday/vdstweek/vdsthour/vdstminute/vdstsecond/vdstmillis/'
                 . 'ldstbias';
@@ -92,7 +92,7 @@ class Horde_ActiveSync_Timezone
         $tz['timezone'] = $tz['bias'];
         $tz['timezonedst'] = $tz['dstbias'];
 
-        if (!self::_isLittleEndian()) {
+        if (!Horde_Mapi::isLittleEndian()) {
             $tz['bias'] = self::_chbo($tz['bias']);
             $tz['stdbias'] = self::_chbo($tz['stdbias']);
             $tz['dstbias'] = self::_chbo($tz['dstbias']);
@@ -109,9 +109,9 @@ class Horde_ActiveSync_Timezone
      * @return string  A base64_encoded ActiveSync Timezone structure suitable
      *                 for transmitting via wbxml.
      */
-    static public function getSyncTZFromOffsets(array $offsets)
+    public static function getSyncTZFromOffsets(array $offsets)
     {
-        if (!self::_isLittleEndian()) {
+        if (!Horde_Mapi::isLittleEndian()) {
             $offsets['bias'] = self::_chbo($offsets['bias']);
             $offsets['stdbias'] = self::_chbo($offsets['stdbias']);
             $offsets['dstbias'] = self::_chbo($offsets['dstbias']);
@@ -133,7 +133,7 @@ class Horde_ActiveSync_Timezone
      *
      * @return array  An offset hash.
      */
-    static public function getOffsetsFromDate(Horde_Date $date)
+    public static function getOffsetsFromDate(Horde_Date $date)
     {
         $offsets = array(
             'bias' => 0,
@@ -184,7 +184,7 @@ class Horde_ActiveSync_Timezone
      *
      * @return array  An array containing the the STD and DST transitions
      */
-    static protected function _getTransitions(DateTimeZone $timezone, Horde_Date $date)
+    protected static function _getTransitions(DateTimeZone $timezone, Horde_Date $date)
     {
 
         $std = $dst = array();
@@ -227,7 +227,7 @@ class Horde_ActiveSync_Timezone
      *
      * @return array  A populated offset hash
      */
-    static protected function _generateOffsetsForTransition(array $offsets, array $transition, $type)
+    protected static function _generateOffsetsForTransition(array $offsets, array $transition, $type)
     {
         // We can't use Horde_Date directly here, since it is unable to
         // properly convert to UTC from local ON the exact hour of a std -> dst
@@ -408,7 +408,7 @@ class Horde_ActiveSync_Timezone
      *                            not occur 5 times
      * @return boolean
      */
-    static protected function _isNthOcurrenceOfWeekdayInMonth($timestamp, $occurence)
+    protected static function _isNthOcurrenceOfWeekdayInMonth($timestamp, $occurence)
     {
         $original = new Horde_Date($timestamp);
         $original->setTimezone('UTC');
@@ -424,18 +424,6 @@ class Horde_ActiveSync_Timezone
        }
     }
 
-    /**
-     * Determine if the current machine is little endian.
-     *
-     * @return boolean  True if endianness is little endian, otherwise false.
-     */
-    static protected function _isLittleEndian() {
-        $testint = 0x00FF;
-        $p = pack('S', $testint);
-
-        return ($testint === current(unpack('v', $p)));
-    }
-
     /**
      * Change the byte order of a number. Used to allow big endian machines to
      * decode the timezone blobs, which are encoded in little endian order.
@@ -444,7 +432,7 @@ class Horde_ActiveSync_Timezone
      *
      * @return integer  The number, in the reverse byte order.
      */
-    static protected function _chbo($num) {
+    protected static function _chbo($num) {
         $u = unpack('l', strrev(pack('l', $num)));
 
         return $u[1];
