@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -19,7 +19,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2014 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2015 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -32,7 +32,7 @@ class Horde_ActiveSync_Utils
      *
      * @return array  The decoded request
      */
-    static public function decodeBase64($uri)
+    public static function decodeBase64($uri)
     {
         $commandMap = array(
                 0  => 'Sync',
@@ -91,40 +91,42 @@ class Horde_ActiveSync_Utils
         while (!feof($stream)) {
             $tag = ord(fread($stream, 1));
             $length = ord(fread($stream, 1));
-            switch ($tag) {
-            case 0:
-                $data = unpack('A' . $length . 'AttName', fread($stream, $length));
-                $results['AttachmentName'] = $data['AttName'];
-                break;
-            case 1:
-                $data = unpack('A' . $length . 'CollId', fread($stream, $length));
-                $results['CollectionId'] = $data['CollId'];
-                break;
-            case 3:
-                $data = unpack('A' . $length . 'ItemId', fread($stream, $length));
-                $results['ItemId'] = $data['ItemId'];
-                break;
-            case 4:
-                $data = unpack('A' . $length . 'Lid', fread($stream, $length));
-                $results['LongId'] = $data['Lid'];
-                break;
-            case 5:
-                $data = unpack('A' . $length . 'Pid', fread($stream, $length));
-                $results['ParentId'] = $data['Pid'];
-                break;
-            case 6:
-                $data = unpack('A' . $length . 'Oc', fread($stream, $length));
-                $results['Occurrence'] = $data['Oc'];
-                break;
-            case 7:
-                $options = ord(fread($stream, 1));
-                $results['SaveInSent'] = !!($options & 0x01);
-                $results['AcceptMultiPart'] = !!($options & 0x02);
-                break;
-            case 8:
-                $data = unpack('A' . $length . 'User', fread($stream, $length));
-                $results['User'] = $data['User'];
-                break;
+            if ($length > 0 || $tag == 7) {
+                switch ($tag) {
+                case 0:
+                    $data = unpack('A' . $length . 'AttName', fread($stream, $length));
+                    $results['AttachmentName'] = $data['AttName'];
+                    break;
+                case 1:
+                    $data = unpack('A' . $length . 'CollId', fread($stream, $length));
+                    $results['CollectionId'] = $data['CollId'];
+                    break;
+                case 3:
+                    $data = unpack('A' . $length . 'ItemId', fread($stream, $length));
+                    $results['ItemId'] = $data['ItemId'];
+                    break;
+                case 4:
+                    $data = unpack('A' . $length . 'Lid', fread($stream, $length));
+                    $results['LongId'] = $data['Lid'];
+                    break;
+                case 5:
+                    $data = unpack('A' . $length . 'Pid', fread($stream, $length));
+                    $results['ParentId'] = $data['Pid'];
+                    break;
+                case 6:
+                    $data = unpack('A' . $length . 'Oc', fread($stream, $length));
+                    $results['Occurrence'] = $data['Oc'];
+                    break;
+                case 7:
+                    $options = ord(fread($stream, 1));
+                    $results['SaveInSent'] = !!($options & 0x01);
+                    $results['AcceptMultiPart'] = !!($options & 0x02);
+                    break;
+                case 8:
+                    $data = unpack('A' . $length . 'User', fread($stream, $length));
+                    $results['User'] = $data['User'];
+                    break;
+                }
             }
         }
 
@@ -141,7 +143,7 @@ class Horde_ActiveSync_Utils
      * @return string  The UID
      * @deprecated  Will be removed in H6. Use Horde_Mapi::getUidFromGoid
      */
-    static public function getUidFromGoid($goid)
+    public static function getUidFromGoid($goid)
     {
         $goid = base64_decode($goid);
 
@@ -174,7 +176,7 @@ class Horde_ActiveSync_Utils
      * @return string  A Base64 encoded GOID
      * @deprecated  Will be removed in H6. Use Horde_Mapi::createGoid
      */
-    static public function createGoid($uid, $options = array())
+    public static function createGoid($uid, $options = array())
     {
         // Bytes 1 - 16 MUST be equal to the GOID identifier:
         $arrayid = '040000008200E00074C5B7101A82E008';
@@ -219,7 +221,7 @@ class Horde_ActiveSync_Utils
      *
      * @return string  A valid UTF-8 encoded string.
      */
-    static public function ensureUtf8($data, $from_charset)
+    public static function ensureUtf8($data, $from_charset)
     {
         $text = Horde_String::convertCharset($data, $from_charset, 'UTF-8');
         if (!Horde_String::validUtf8($text)) {
@@ -236,7 +238,7 @@ class Horde_ActiveSync_Utils
                 }
             }
             // Invalid UTF-8 still found. Strip out non 7-bit characters, or if
-            // that fails, force a conersion to UTF-8 as a last resort. Need
+            // that fails, force a conversion to UTF-8 as a last resort. Need
             // to break string into smaller chunks to avoid hitting
             // https://bugs.php.net/bug.php?id=37793
             $chunk_size = 4000;
@@ -262,7 +264,7 @@ class Horde_ActiveSync_Utils
      *
      * @return string|boolean  The stripped string, or false if failed.
      */
-    static protected function _stripNon7BitChars($text)
+    protected static function _stripNon7BitChars($text)
     {
         return preg_replace('/[^\x09\x0A\x0D\x20-\x7E]/', '', $text);
     }
