@@ -20,7 +20,7 @@ class Horde_ActiveSync_DeviceTest extends Horde_Test_Case
         $device = new Horde_ActiveSync_Device($state, $fixture);
         $this->assertEquals(7, $device->getMajorVersion());
         $this->assertEquals(0, $device->getMinorVersion());
-        $this->assertEquals(Horde_ActiveSync_Device::TYPE_IPOD, strtolower($device->deviceType));
+        $this->assertEquals(Horde_ActiveSync_Device::TYPE_IPOD, Horde_String::lower($device->deviceType));
         $this->assertEquals(Horde_ActiveSync_Device::MULTIPLEX_NOTES, $device->multiplex);
 
         $fixture = array(
@@ -30,7 +30,7 @@ class Horde_ActiveSync_DeviceTest extends Horde_Test_Case
         $device = new Horde_ActiveSync_Device($state, $fixture);
         $this->assertEquals(6, $device->getMajorVersion());
         $this->assertEquals(1, $device->getMinorVersion());
-        $this->assertEquals(Horde_ActiveSync_Device::TYPE_IPHONE, strtolower($device->deviceType));
+        $this->assertEquals(Horde_ActiveSync_Device::TYPE_IPHONE, Horde_String::lower($device->deviceType));
         $this->assertEquals(Horde_ActiveSync_Device::MULTIPLEX_NOTES, $device->multiplex);
 
         // Old Android.
@@ -40,8 +40,8 @@ class Horde_ActiveSync_DeviceTest extends Horde_Test_Case
         $device = new Horde_ActiveSync_Device($state, $fixture);
         $this->assertEquals(0, $device->getMajorVersion());
         $this->assertEquals(3, $device->getMinorVersion());
-        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, strtolower($device->deviceType));
-        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, strtolower($device->clientType));
+        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, Horde_String::lower($device->deviceType));
+        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, Horde_String::lower($device->clientType));
         $this->assertEquals(Horde_ActiveSync_Device::MULTIPLEX_CONTACTS |
                     Horde_ActiveSync_Device::MULTIPLEX_CALENDAR |
                     Horde_ActiveSync_Device::MULTIPLEX_NOTES |
@@ -54,8 +54,8 @@ class Horde_ActiveSync_DeviceTest extends Horde_Test_Case
         $device = new Horde_ActiveSync_Device($state, $fixture);
         $this->assertEquals(7, $device->getMajorVersion());
         $this->assertEquals(1, $device->getMinorVersion());
-        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, strtolower($device->deviceType));
-        $this->assertEquals(Horde_ActiveSync_Device::TYPE_TOUCHDOWN, strtolower($device->clientType));
+        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, Horde_String::lower($device->deviceType));
+        $this->assertEquals(Horde_ActiveSync_Device::TYPE_TOUCHDOWN, Horde_String::lower($device->clientType));
         $this->assertEquals(0, $device->multiplex);
 
         // Not-so-old-but-still-old Android.
@@ -65,8 +65,8 @@ class Horde_ActiveSync_DeviceTest extends Horde_Test_Case
         $device = new Horde_ActiveSync_Device($state, $fixture);
         $this->assertEquals(2, $device->getMajorVersion());
         $this->assertEquals(1707, $device->getMinorVersion());
-        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, strtolower($device->deviceType));
-        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, strtolower($device->clientType));
+        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, Horde_String::lower($device->deviceType));
+        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, Horde_String::lower($device->clientType));
         $this->assertEquals(15, $device->multiplex);
 
 
@@ -79,8 +79,8 @@ class Horde_ActiveSync_DeviceTest extends Horde_Test_Case
         $device = new Horde_ActiveSync_Device($state, $fixture);
         $this->assertEquals(4, $device->getMajorVersion());
         $this->assertEquals(4, $device->getMinorVersion());
-        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, strtolower($device->deviceType));
-        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, strtolower($device->clientType));
+        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, Horde_String::lower($device->deviceType));
+        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, Horde_String::lower($device->clientType));
         $this->assertEquals(13, $device->multiplex);
 
         // Devices like this (from a Note 3) we simply can't sniff multiplex for
@@ -89,13 +89,14 @@ class Horde_ActiveSync_DeviceTest extends Horde_Test_Case
         $fixture = array(
             'deviceType' => 'SAMSUNGSMN900V',
             'userAgent' => 'SAMSUNG-SM-N900V/101.403',
-            'properties' => array(Horde_ActiveSync_Device::OS => 'Android 4.4.2.N900VVRUCNC4')
+            'properties' => array(Horde_ActiveSync_Device::OS => 'Android')
         );
         $device = new Horde_ActiveSync_Device($state, $fixture);
-        $this->assertEquals(4, $device->getMajorVersion());
-        $this->assertEquals(4, $device->getMinorVersion());
-        $this->assertEquals('samsungsmn900v', strtolower($device->deviceType));
-        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, strtolower($device->clientType));
+        // These are useless values, but still tests the reliability of the code
+        $this->assertEquals(101, $device->getMajorVersion());
+        $this->assertEquals(403, $device->getMinorVersion());
+        $this->assertEquals('samsungsmn900v', Horde_String::lower($device->deviceType));
+        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, Horde_String::lower($device->clientType));
         $this->assertEquals(15, $device->multiplex);
 
         // Nine (From Note 3 running 4.4.2).
@@ -108,9 +109,22 @@ class Horde_ActiveSync_DeviceTest extends Horde_Test_Case
         $device->id = '6E696E656331393035333833303331';
 
         $this->assertEquals(4, $device->getMajorVersion());
-        $this->assertEquals('android', strtolower($device->deviceType));
-        $this->assertEquals(Horde_ActiveSync_Device::TYPE_NINE, strtolower($device->clientType));
+        $this->assertEquals('android', Horde_String::lower($device->deviceType));
+        $this->assertEquals(Horde_ActiveSync_Device::TYPE_NINE, Horde_String::lower($device->clientType));
         $this->assertEquals(0, $device->multiplex);
+
+        // HTCOneMini2
+        $fixture = array(
+            'userAgent' => 'HTC', // Don't think this matters here.
+            'deviceType' => 'HTCOnemini2',
+            'properties' => array(Horde_ActiveSync_Device::OS => 'Android 4.4.2', Horde_ActiveSync_Device::MODEL => 'HTCOnemini2')
+        );
+        $device = new Horde_ActiveSync_Device($state, $fixture);
+        $this->assertEquals(4, $device->getMajorVersion());
+        $this->assertEquals(4, $device->getMinorVersion());
+        $this->assertEquals('htconemini2', Horde_String::lower($device->deviceType));
+        $this->assertEquals(Horde_ActiveSync_Device::TYPE_ANDROID, Horde_String::lower($device->clientType));
+        $this->assertEquals(15, $device->multiplex);
     }
 
     public function testPoomContactsDate()
@@ -131,7 +145,7 @@ class Horde_ActiveSync_DeviceTest extends Horde_Test_Case
         $fixture = array(
             'deviceType' => 'iPhone',
             'userAgent' => 'Apple-iPhone4C1/1002.329',
-            'properties' => array(Horde_ActiveSync_Device::OS => ' iOS 6.1.3 10B329'));
+            'properties' => array(Horde_ActiveSync_Device::OS => 'iOS 6.1.3 10B329'));
         $device = new Horde_ActiveSync_Device($state, $fixture);
         $date = new Horde_Date('1970-03-20');
         $bday = $device->normalizePoomContactsDates($date, true);
@@ -141,6 +155,17 @@ class Horde_ActiveSync_DeviceTest extends Horde_Test_Case
         $bday = $device->normalizePoomContactsDates($date);
         $this->assertEquals('1970-03-20 00:00:00', (string)$bday->setTimezone('America/New_York'));
 
+        // Try a positive UTC offset timezone
+        date_default_timezone_set('Europe/Berlin');
+        $fixture = array(
+            'deviceType' => 'iPhone',
+            'userAgent' => 'Apple-iPhone4C1/1104.201',
+            'properties' => array(Horde_ActiveSync_Device::OS => 'iOS 7.1.1 11D201'));
+        $device = new Horde_ActiveSync_Device($state, $fixture);
+        $date = new Horde_Date('1966-07-22T23:00:00.000Z');
+        $bday = $device->normalizePoomContactsDates($date);
+        $bday->setTimezone(date_default_timezone_get());
+        $this->assertEquals('1966-07-23', $bday->format('Y-m-d'));
 
         // Android
         date_default_timezone_set('Pacific/Honolulu');
@@ -150,6 +175,7 @@ class Horde_ActiveSync_DeviceTest extends Horde_Test_Case
         $bday = $device->normalizePoomContactsDates($date);
         $this->assertEquals('2003-09-24', $bday->setTimezone('Pacific/Honolulu')->format('Y-m-d'));
 
+        // Note 3
         date_default_timezone_set('America/Chicago');
         $fixture = array(
             'deviceType' => 'SAMSUNGSMN900V',
@@ -159,7 +185,7 @@ class Horde_ActiveSync_DeviceTest extends Horde_Test_Case
         $device = new Horde_ActiveSync_Device($state, $fixture);
         $date = new Horde_Date('1970-03-20');
         $bday = $device->normalizePoomContactsDates($date, true);
-        $this->assertEquals('1970-03-20 08:00:00', (string)$bday);
+        $this->assertEquals('1970-03-20 00:00:00', (string)$bday);
 
         $fixture = array(
             'deviceType' => 'Android',
